//! This file was generated automatically by the Snowball to Rust compiler
//! http://snowballstem.org/

#![allow(non_upper_case_globals)]
#![allow(non_snake_case)]
#![allow(unused_variables)]
#![allow(unused_mut)]
use snowball::SnowballEnv;
use snowball::Among;

static A_0: &'static [Among<Context>; 3] = &[
    Among("", -1, 3, None),
    Among("\u{00E3}", 0, 1, None),
    Among("\u{00F5}", 0, 2, None),
];

static A_1: &'static [Among<Context>; 3] = &[
    Among("", -1, 3, None),
    Among("a~", 0, 1, None),
    Among("o~", 0, 2, None),
];

static A_2: &'static [Among<Context>; 4] = &[
    Among("ic", -1, -1, None),
    Among("ad", -1, -1, None),
    Among("os", -1, -1, None),
    Among("iv", -1, 1, None),
];

static A_3: &'static [Among<Context>; 3] = &[
    Among("ante", -1, 1, None),
    Among("avel", -1, 1, None),
    Among("\u{00ED}vel", -1, 1, None),
];

static A_4: &'static [Among<Context>; 3] = &[
    Among("ic", -1, 1, None),
    Among("abil", -1, 1, None),
    Among("iv", -1, 1, None),
];

static A_5: &'static [Among<Context>; 45] = &[
    Among("ica", -1, 1, None),
    Among("\u{00E2}ncia", -1, 1, None),
    Among("\u{00EA}ncia", -1, 4, None),
    Among("logia", -1, 2, None),
    Among("ira", -1, 9, None),
    Among("adora", -1, 1, None),
    Among("osa", -1, 1, None),
    Among("ista", -1, 1, None),
    Among("iva", -1, 8, None),
    Among("eza", -1, 1, None),
    Among("idade", -1, 7, None),
    Among("ante", -1, 1, None),
    Among("mente", -1, 6, None),
    Among("amente", 12, 5, None),
    Among("\u{00E1}vel", -1, 1, None),
    Among("\u{00ED}vel", -1, 1, None),
    Among("ico", -1, 1, None),
    Among("ismo", -1, 1, None),
    Among("oso", -1, 1, None),
    Among("amento", -1, 1, None),
    Among("imento", -1, 1, None),
    Among("ivo", -1, 8, None),
    Among("a\u{00E7}a~o", -1, 1, None),
    Among("u\u{00E7}a~o", -1, 3, None),
    Among("ador", -1, 1, None),
    Among("icas", -1, 1, None),
    Among("\u{00EA}ncias", -1, 4, None),
    Among("logias", -1, 2, None),
    Among("iras", -1, 9, None),
    Among("adoras", -1, 1, None),
    Among("osas", -1, 1, None),
    Among("istas", -1, 1, None),
    Among("ivas", -1, 8, None),
    Among("ezas", -1, 1, None),
    Among("idades", -1, 7, None),
    Among("adores", -1, 1, None),
    Among("antes", -1, 1, None),
    Among("a\u{00E7}o~es", -1, 1, None),
    Among("u\u{00E7}o~es", -1, 3, None),
    Among("icos", -1, 1, None),
    Among("ismos", -1, 1, None),
    Among("osos", -1, 1, None),
    Among("amentos", -1, 1, None),
    Among("imentos", -1, 1, None),
    Among("ivos", -1, 8, None),
];

static A_6: &'static [Among<Context>; 120] = &[
    Among("ada", -1, 1, None),
    Among("ida", -1, 1, None),
    Among("ia", -1, 1, None),
    Among("aria", 2, 1, None),
    Among("eria", 2, 1, None),
    Among("iria", 2, 1, None),
    Among("ara", -1, 1, None),
    Among("era", -1, 1, None),
    Among("ira", -1, 1, None),
    Among("ava", -1, 1, None),
    Among("asse", -1, 1, None),
    Among("esse", -1, 1, None),
    Among("isse", -1, 1, None),
    Among("aste", -1, 1, None),
    Among("este", -1, 1, None),
    Among("iste", -1, 1, None),
    Among("ei", -1, 1, None),
    Among("arei", 16, 1, None),
    Among("erei", 16, 1, None),
    Among("irei", 16, 1, None),
    Among("am", -1, 1, None),
    Among("iam", 20, 1, None),
    Among("ariam", 21, 1, None),
    Among("eriam", 21, 1, None),
    Among("iriam", 21, 1, None),
    Among("aram", 20, 1, None),
    Among("eram", 20, 1, None),
    Among("iram", 20, 1, None),
    Among("avam", 20, 1, None),
    Among("em", -1, 1, None),
    Among("arem", 29, 1, None),
    Among("erem", 29, 1, None),
    Among("irem", 29, 1, None),
    Among("assem", 29, 1, None),
    Among("essem", 29, 1, None),
    Among("issem", 29, 1, None),
    Among("ado", -1, 1, None),
    Among("ido", -1, 1, None),
    Among("ando", -1, 1, None),
    Among("endo", -1, 1, None),
    Among("indo", -1, 1, None),
    Among("ara~o", -1, 1, None),
    Among("era~o", -1, 1, None),
    Among("ira~o", -1, 1, None),
    Among("ar", -1, 1, None),
    Among("er", -1, 1, None),
    Among("ir", -1, 1, None),
    Among("as", -1, 1, None),
    Among("adas", 47, 1, None),
    Among("idas", 47, 1, None),
    Among("ias", 47, 1, None),
    Among("arias", 50, 1, None),
    Among("erias", 50, 1, None),
    Among("irias", 50, 1, None),
    Among("aras", 47, 1, None),
    Among("eras", 47, 1, None),
    Among("iras", 47, 1, None),
    Among("avas", 47, 1, None),
    Among("es", -1, 1, None),
    Among("ardes", 58, 1, None),
    Among("erdes", 58, 1, None),
    Among("irdes", 58, 1, None),
    Among("ares", 58, 1, None),
    Among("eres", 58, 1, None),
    Among("ires", 58, 1, None),
    Among("asses", 58, 1, None),
    Among("esses", 58, 1, None),
    Among("isses", 58, 1, None),
    Among("astes", 58, 1, None),
    Among("estes", 58, 1, None),
    Among("istes", 58, 1, None),
    Among("is", -1, 1, None),
    Among("ais", 71, 1, None),
    Among("eis", 71, 1, None),
    Among("areis", 73, 1, None),
    Among("ereis", 73, 1, None),
    Among("ireis", 73, 1, None),
    Among("\u{00E1}reis", 73, 1, None),
    Among("\u{00E9}reis", 73, 1, None),
    Among("\u{00ED}reis", 73, 1, None),
    Among("\u{00E1}sseis", 73, 1, None),
    Among("\u{00E9}sseis", 73, 1, None),
    Among("\u{00ED}sseis", 73, 1, None),
    Among("\u{00E1}veis", 73, 1, None),
    Among("\u{00ED}eis", 73, 1, None),
    Among("ar\u{00ED}eis", 84, 1, None),
    Among("er\u{00ED}eis", 84, 1, None),
    Among("ir\u{00ED}eis", 84, 1, None),
    Among("ados", -1, 1, None),
    Among("idos", -1, 1, None),
    Among("amos", -1, 1, None),
    Among("\u{00E1}ramos", 90, 1, None),
    Among("\u{00E9}ramos", 90, 1, None),
    Among("\u{00ED}ramos", 90, 1, None),
    Among("\u{00E1}vamos", 90, 1, None),
    Among("\u{00ED}amos", 90, 1, None),
    Among("ar\u{00ED}amos", 95, 1, None),
    Among("er\u{00ED}amos", 95, 1, None),
    Among("ir\u{00ED}amos", 95, 1, None),
    Among("emos", -1, 1, None),
    Among("aremos", 99, 1, None),
    Among("eremos", 99, 1, None),
    Among("iremos", 99, 1, None),
    Among("\u{00E1}ssemos", 99, 1, None),
    Among("\u{00EA}ssemos", 99, 1, None),
    Among("\u{00ED}ssemos", 99, 1, None),
    Among("imos", -1, 1, None),
    Among("armos", -1, 1, None),
    Among("ermos", -1, 1, None),
    Among("irmos", -1, 1, None),
    Among("\u{00E1}mos", -1, 1, None),
    Among("ar\u{00E1}s", -1, 1, None),
    Among("er\u{00E1}s", -1, 1, None),
    Among("ir\u{00E1}s", -1, 1, None),
    Among("eu", -1, 1, None),
    Among("iu", -1, 1, None),
    Among("ou", -1, 1, None),
    Among("ar\u{00E1}", -1, 1, None),
    Among("er\u{00E1}", -1, 1, None),
    Among("ir\u{00E1}", -1, 1, None),
];

static A_7: &'static [Among<Context>; 7] = &[
    Among("a", -1, 1, None),
    Among("i", -1, 1, None),
    Among("o", -1, 1, None),
    Among("os", -1, 1, None),
    Among("\u{00E1}", -1, 1, None),
    Among("\u{00ED}", -1, 1, None),
    Among("\u{00F3}", -1, 1, None),
];

static A_8: &'static [Among<Context>; 4] = &[
    Among("e", -1, 1, None),
    Among("\u{00E7}", -1, 2, None),
    Among("\u{00E9}", -1, 1, None),
    Among("\u{00EA}", -1, 1, None),
];

static G_v: &'static [u8; 20] = &[17, 65, 16, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 19, 12, 2];

#[derive(Clone)]
struct Context {
    i_p2: usize,
    i_p1: usize,
    i_pV: usize,
}

fn r_prelude(env: &mut SnowballEnv, context: &mut Context) -> bool {
    let mut among_var;
    // repeat, line 36
    'replab0: loop{
        let v_1 = env.cursor;
        'lab1: for _ in 0..1 {
            // (, line 36
            // [, line 37
            env.bra = env.cursor;
            // substring, line 37
            among_var = env.find_among(A_0, context);
            if among_var == 0 {
                break 'lab1;
            }
            // ], line 37
            env.ket = env.cursor;
            if among_var == 0 {
                break 'lab1;
            } else if among_var == 1 {
                // (, line 38
                // <-, line 38
                if !env.slice_from("a~") {
                    return false;
                }
            } else if among_var == 2 {
                // (, line 39
                // <-, line 39
                if !env.slice_from("o~") {
                    return false;
                }
            } else if among_var == 3 {
                // (, line 40
                // next, line 40
                if env.cursor >= env.limit {
                    break 'lab1;
                }
                env.next_char();
            }
            continue 'replab0;
        }
        env.cursor = v_1;
        break 'replab0;
    }
    return true;
}

fn r_mark_regions(env: &mut SnowballEnv, context: &mut Context) -> bool {
    // (, line 44
    context.i_pV = env.limit;
    context.i_p1 = env.limit;
    context.i_p2 = env.limit;
    // do, line 50
    let v_1 = env.cursor;
    'lab0: loop {
        // (, line 50
        // or, line 52
        'lab1: loop {
            let v_2 = env.cursor;
            'lab2: loop {
                // (, line 51
                if !env.in_grouping(G_v, 97, 250) {
                    break 'lab2;
                }
                // or, line 51
                'lab3: loop {
                    let v_3 = env.cursor;
                    'lab4: loop {
                        // (, line 51
                        if !env.out_grouping(G_v, 97, 250) {
                            break 'lab4;
                        }
                        // gopast, line 51
                        'golab5: loop {
                            'lab6: loop {
                                if !env.in_grouping(G_v, 97, 250) {
                                    break 'lab6;
                                }
                                break 'golab5;
                            }
                            if env.cursor >= env.limit {
                                break 'lab4;
                            }
                            env.next_char();
                        }
                        break 'lab3;
                    }
                    env.cursor = v_3;
                    // (, line 51
                    if !env.in_grouping(G_v, 97, 250) {
                        break 'lab2;
                    }
                    // gopast, line 51
                    'golab7: loop {
                        'lab8: loop {
                            if !env.out_grouping(G_v, 97, 250) {
                                break 'lab8;
                            }
                            break 'golab7;
                        }
                        if env.cursor >= env.limit {
                            break 'lab2;
                        }
                        env.next_char();
                    }
                    break 'lab3;
                }
                break 'lab1;
            }
            env.cursor = v_2;
            // (, line 53
            if !env.out_grouping(G_v, 97, 250) {
                break 'lab0;
            }
            // or, line 53
            'lab9: loop {
                let v_6 = env.cursor;
                'lab10: loop {
                    // (, line 53
                    if !env.out_grouping(G_v, 97, 250) {
                        break 'lab10;
                    }
                    // gopast, line 53
                    'golab11: loop {
                        'lab12: loop {
                            if !env.in_grouping(G_v, 97, 250) {
                                break 'lab12;
                            }
                            break 'golab11;
                        }
                        if env.cursor >= env.limit {
                            break 'lab10;
                        }
                        env.next_char();
                    }
                    break 'lab9;
                }
                env.cursor = v_6;
                // (, line 53
                if !env.in_grouping(G_v, 97, 250) {
                    break 'lab0;
                }
                // next, line 53
                if env.cursor >= env.limit {
                    break 'lab0;
                }
                env.next_char();
                break 'lab9;
            }
            break 'lab1;
        }
        // setmark pV, line 54
        context.i_pV = env.cursor;
        break 'lab0;
    }
    env.cursor = v_1;
    // do, line 56
    let v_8 = env.cursor;
    'lab13: loop {
        // (, line 56
        // gopast, line 57
        'golab14: loop {
            'lab15: loop {
                if !env.in_grouping(G_v, 97, 250) {
                    break 'lab15;
                }
                break 'golab14;
            }
            if env.cursor >= env.limit {
                break 'lab13;
            }
            env.next_char();
        }
        // gopast, line 57
        'golab16: loop {
            'lab17: loop {
                if !env.out_grouping(G_v, 97, 250) {
                    break 'lab17;
                }
                break 'golab16;
            }
            if env.cursor >= env.limit {
                break 'lab13;
            }
            env.next_char();
        }
        // setmark p1, line 57
        context.i_p1 = env.cursor;
        // gopast, line 58
        'golab18: loop {
            'lab19: loop {
                if !env.in_grouping(G_v, 97, 250) {
                    break 'lab19;
                }
                break 'golab18;
            }
            if env.cursor >= env.limit {
                break 'lab13;
            }
            env.next_char();
        }
        // gopast, line 58
        'golab20: loop {
            'lab21: loop {
                if !env.out_grouping(G_v, 97, 250) {
                    break 'lab21;
                }
                break 'golab20;
            }
            if env.cursor >= env.limit {
                break 'lab13;
            }
            env.next_char();
        }
        // setmark p2, line 58
        context.i_p2 = env.cursor;
        break 'lab13;
    }
    env.cursor = v_8;
    return true;
}

fn r_postlude(env: &mut SnowballEnv, context: &mut Context) -> bool {
    let mut among_var;
    // repeat, line 62
    'replab0: loop{
        let v_1 = env.cursor;
        'lab1: for _ in 0..1 {
            // (, line 62
            // [, line 63
            env.bra = env.cursor;
            // substring, line 63
            among_var = env.find_among(A_1, context);
            if among_var == 0 {
                break 'lab1;
            }
            // ], line 63
            env.ket = env.cursor;
            if among_var == 0 {
                break 'lab1;
            } else if among_var == 1 {
                // (, line 64
                // <-, line 64
                if !env.slice_from("\u{00E3}") {
                    return false;
                }
            } else if among_var == 2 {
                // (, line 65
                // <-, line 65
                if !env.slice_from("\u{00F5}") {
                    return false;
                }
            } else if among_var == 3 {
                // (, line 66
                // next, line 66
                if env.cursor >= env.limit {
                    break 'lab1;
                }
                env.next_char();
            }
            continue 'replab0;
        }
        env.cursor = v_1;
        break 'replab0;
    }
    return true;
}

fn r_RV(env: &mut SnowballEnv, context: &mut Context) -> bool {
    if !(context.i_pV <= env.cursor){
        return false;
    }
    return true;
}

fn r_R1(env: &mut SnowballEnv, context: &mut Context) -> bool {
    if !(context.i_p1 <= env.cursor){
        return false;
    }
    return true;
}

fn r_R2(env: &mut SnowballEnv, context: &mut Context) -> bool {
    if !(context.i_p2 <= env.cursor){
        return false;
    }
    return true;
}

fn r_standard_suffix(env: &mut SnowballEnv, context: &mut Context) -> bool {
    let mut among_var;
    // (, line 76
    // [, line 77
    env.ket = env.cursor;
    // substring, line 77
    among_var = env.find_among_b(A_5, context);
    if among_var == 0 {
        return false;
    }
    // ], line 77
    env.bra = env.cursor;
    if among_var == 0 {
        return false;
    } else if among_var == 1 {
        // (, line 92
        // call R2, line 93
        if !r_R2(env, context) {
            return false;
        }
        // delete, line 93
        if !env.slice_del() {
            return false;
        }
    } else if among_var == 2 {
        // (, line 97
        // call R2, line 98
        if !r_R2(env, context) {
            return false;
        }
        // <-, line 98
        if !env.slice_from("log") {
            return false;
        }
    } else if among_var == 3 {
        // (, line 101
        // call R2, line 102
        if !r_R2(env, context) {
            return false;
        }
        // <-, line 102
        if !env.slice_from("u") {
            return false;
        }
    } else if among_var == 4 {
        // (, line 105
        // call R2, line 106
        if !r_R2(env, context) {
            return false;
        }
        // <-, line 106
        if !env.slice_from("ente") {
            return false;
        }
    } else if among_var == 5 {
        // (, line 109
        // call R1, line 110
        if !r_R1(env, context) {
            return false;
        }
        // delete, line 110
        if !env.slice_del() {
            return false;
        }
        // try, line 111
        let v_1 = env.limit - env.cursor;
        'lab0: loop {
            // (, line 111
            // [, line 112
            env.ket = env.cursor;
            // substring, line 112
            among_var = env.find_among_b(A_2, context);
            if among_var == 0 {
                env.cursor = env.limit - v_1;
                break 'lab0;
            }
            // ], line 112
            env.bra = env.cursor;
            // call R2, line 112
            if !r_R2(env, context) {
                env.cursor = env.limit - v_1;
                break 'lab0;
            }
            // delete, line 112
            if !env.slice_del() {
                return false;
            }
            if among_var == 0 {
                env.cursor = env.limit - v_1;
                break 'lab0;
            } else if among_var == 1 {
                // (, line 113
                // [, line 113
                env.ket = env.cursor;
                // literal, line 113
                if !env.eq_s_b(&"at") {
                    env.cursor = env.limit - v_1;
                    break 'lab0;
                }
                // ], line 113
                env.bra = env.cursor;
                // call R2, line 113
                if !r_R2(env, context) {
                    env.cursor = env.limit - v_1;
                    break 'lab0;
                }
                // delete, line 113
                if !env.slice_del() {
                    return false;
                }
            }
            break 'lab0;
        }
    } else if among_var == 6 {
        // (, line 121
        // call R2, line 122
        if !r_R2(env, context) {
            return false;
        }
        // delete, line 122
        if !env.slice_del() {
            return false;
        }
        // try, line 123
        let v_2 = env.limit - env.cursor;
        'lab1: loop {
            // (, line 123
            // [, line 124
            env.ket = env.cursor;
            // substring, line 124
            among_var = env.find_among_b(A_3, context);
            if among_var == 0 {
                env.cursor = env.limit - v_2;
                break 'lab1;
            }
            // ], line 124
            env.bra = env.cursor;
            if among_var == 0 {
                env.cursor = env.limit - v_2;
                break 'lab1;
            } else if among_var == 1 {
                // (, line 127
                // call R2, line 127
                if !r_R2(env, context) {
                    env.cursor = env.limit - v_2;
                    break 'lab1;
                }
                // delete, line 127
                if !env.slice_del() {
                    return false;
                }
            }
            break 'lab1;
        }
    } else if among_var == 7 {
        // (, line 133
        // call R2, line 134
        if !r_R2(env, context) {
            return false;
        }
        // delete, line 134
        if !env.slice_del() {
            return false;
        }
        // try, line 135
        let v_3 = env.limit - env.cursor;
        'lab2: loop {
            // (, line 135
            // [, line 136
            env.ket = env.cursor;
            // substring, line 136
            among_var = env.find_among_b(A_4, context);
            if among_var == 0 {
                env.cursor = env.limit - v_3;
                break 'lab2;
            }
            // ], line 136
            env.bra = env.cursor;
            if among_var == 0 {
                env.cursor = env.limit - v_3;
                break 'lab2;
            } else if among_var == 1 {
                // (, line 139
                // call R2, line 139
                if !r_R2(env, context) {
                    env.cursor = env.limit - v_3;
                    break 'lab2;
                }
                // delete, line 139
                if !env.slice_del() {
                    return false;
                }
            }
            break 'lab2;
        }
    } else if among_var == 8 {
        // (, line 145
        // call R2, line 146
        if !r_R2(env, context) {
            return false;
        }
        // delete, line 146
        if !env.slice_del() {
            return false;
        }
        // try, line 147
        let v_4 = env.limit - env.cursor;
        'lab3: loop {
            // (, line 147
            // [, line 148
            env.ket = env.cursor;
            // literal, line 148
            if !env.eq_s_b(&"at") {
                env.cursor = env.limit - v_4;
                break 'lab3;
            }
            // ], line 148
            env.bra = env.cursor;
            // call R2, line 148
            if !r_R2(env, context) {
                env.cursor = env.limit - v_4;
                break 'lab3;
            }
            // delete, line 148
            if !env.slice_del() {
                return false;
            }
            break 'lab3;
        }
    } else if among_var == 9 {
        // (, line 152
        // call RV, line 153
        if !r_RV(env, context) {
            return false;
        }
        // literal, line 153
        if !env.eq_s_b(&"e") {
            return false;
        }
        // <-, line 154
        if !env.slice_from("ir") {
            return false;
        }
    }
    return true;
}

fn r_verb_suffix(env: &mut SnowballEnv, context: &mut Context) -> bool {
    let mut among_var;
    // setlimit, line 159
    let v_1 = env.limit - env.cursor;
    // tomark, line 159
    if env.cursor < context.i_pV {
        return false;
    }
    env.cursor = context.i_pV;
    let v_2 = env.limit_backward;
    env.limit_backward = env.cursor;
    env.cursor = env.limit - v_1;
    // (, line 159
    // [, line 160
    env.ket = env.cursor;
    // substring, line 160
    among_var = env.find_among_b(A_6, context);
    if among_var == 0 {
        env.limit_backward = v_2;
        return false;
    }
    // ], line 160
    env.bra = env.cursor;
    if among_var == 0 {
        env.limit_backward = v_2;
        return false;
    } else if among_var == 1 {
        // (, line 179
        // delete, line 179
        if !env.slice_del() {
            return false;
        }
    }
    env.limit_backward = v_2;
    return true;
}

fn r_residual_suffix(env: &mut SnowballEnv, context: &mut Context) -> bool {
    let mut among_var;
    // (, line 183
    // [, line 184
    env.ket = env.cursor;
    // substring, line 184
    among_var = env.find_among_b(A_7, context);
    if among_var == 0 {
        return false;
    }
    // ], line 184
    env.bra = env.cursor;
    if among_var == 0 {
        return false;
    } else if among_var == 1 {
        // (, line 187
        // call RV, line 187
        if !r_RV(env, context) {
            return false;
        }
        // delete, line 187
        if !env.slice_del() {
            return false;
        }
    }
    return true;
}

fn r_residual_form(env: &mut SnowballEnv, context: &mut Context) -> bool {
    let mut among_var;
    // (, line 191
    // [, line 192
    env.ket = env.cursor;
    // substring, line 192
    among_var = env.find_among_b(A_8, context);
    if among_var == 0 {
        return false;
    }
    // ], line 192
    env.bra = env.cursor;
    if among_var == 0 {
        return false;
    } else if among_var == 1 {
        // (, line 194
        // call RV, line 194
        if !r_RV(env, context) {
            return false;
        }
        // delete, line 194
        if !env.slice_del() {
            return false;
        }
        // [, line 194
        env.ket = env.cursor;
        // or, line 194
        'lab0: loop {
            let v_1 = env.limit - env.cursor;
            'lab1: loop {
                // (, line 194
                // literal, line 194
                if !env.eq_s_b(&"u") {
                    break 'lab1;
                }
                // ], line 194
                env.bra = env.cursor;
                // test, line 194
                let v_2 = env.limit - env.cursor;
                // literal, line 194
                if !env.eq_s_b(&"g") {
                    break 'lab1;
                }
                env.cursor = env.limit - v_2;
                break 'lab0;
            }
            env.cursor = env.limit - v_1;
            // (, line 195
            // literal, line 195
            if !env.eq_s_b(&"i") {
                return false;
            }
            // ], line 195
            env.bra = env.cursor;
            // test, line 195
            let v_3 = env.limit - env.cursor;
            // literal, line 195
            if !env.eq_s_b(&"c") {
                return false;
            }
            env.cursor = env.limit - v_3;
            break 'lab0;
        }
        // call RV, line 195
        if !r_RV(env, context) {
            return false;
        }
        // delete, line 195
        if !env.slice_del() {
            return false;
        }
    } else if among_var == 2 {
        // (, line 196
        // <-, line 196
        if !env.slice_from("c") {
            return false;
        }
    }
    return true;
}

pub fn stem(env: &mut SnowballEnv) -> bool {
    let mut context = &mut Context {
        i_p2: 0,
        i_p1: 0,
        i_pV: 0,
    };
    // (, line 201
    // do, line 202
    let v_1 = env.cursor;
    'lab0: loop {
        // call prelude, line 202
        if !r_prelude(env, context) {
            break 'lab0;
        }
        break 'lab0;
    }
    env.cursor = v_1;
    // do, line 203
    let v_2 = env.cursor;
    'lab1: loop {
        // call mark_regions, line 203
        if !r_mark_regions(env, context) {
            break 'lab1;
        }
        break 'lab1;
    }
    env.cursor = v_2;
    // backwards, line 204
    env.limit_backward = env.cursor;
    env.cursor = env.limit;
    // (, line 204
    // do, line 205
    let v_3 = env.limit - env.cursor;
    'lab2: loop {
        // (, line 205
        // or, line 209
        'lab3: loop {
            let v_4 = env.limit - env.cursor;
            'lab4: loop {
                // (, line 206
                // and, line 207
                let v_5 = env.limit - env.cursor;
                // (, line 206
                // or, line 206
                'lab5: loop {
                    let v_6 = env.limit - env.cursor;
                    'lab6: loop {
                        // call standard_suffix, line 206
                        if !r_standard_suffix(env, context) {
                            break 'lab6;
                        }
                        break 'lab5;
                    }
                    env.cursor = env.limit - v_6;
                    // call verb_suffix, line 206
                    if !r_verb_suffix(env, context) {
                        break 'lab4;
                    }
                    break 'lab5;
                }
                env.cursor = env.limit - v_5;
                // do, line 207
                let v_7 = env.limit - env.cursor;
                'lab7: loop {
                    // (, line 207
                    // [, line 207
                    env.ket = env.cursor;
                    // literal, line 207
                    if !env.eq_s_b(&"i") {
                        break 'lab7;
                    }
                    // ], line 207
                    env.bra = env.cursor;
                    // test, line 207
                    let v_8 = env.limit - env.cursor;
                    // literal, line 207
                    if !env.eq_s_b(&"c") {
                        break 'lab7;
                    }
                    env.cursor = env.limit - v_8;
                    // call RV, line 207
                    if !r_RV(env, context) {
                        break 'lab7;
                    }
                    // delete, line 207
                    if !env.slice_del() {
                        return false;
                    }
                    break 'lab7;
                }
                env.cursor = env.limit - v_7;
                break 'lab3;
            }
            env.cursor = env.limit - v_4;
            // call residual_suffix, line 209
            if !r_residual_suffix(env, context) {
                break 'lab2;
            }
            break 'lab3;
        }
        break 'lab2;
    }
    env.cursor = env.limit - v_3;
    // do, line 211
    let v_9 = env.limit - env.cursor;
    'lab8: loop {
        // call residual_form, line 211
        if !r_residual_form(env, context) {
            break 'lab8;
        }
        break 'lab8;
    }
    env.cursor = env.limit - v_9;
    env.cursor = env.limit_backward;
    // do, line 213
    let v_10 = env.cursor;
    'lab9: loop {
        // call postlude, line 213
        if !r_postlude(env, context) {
            break 'lab9;
        }
        break 'lab9;
    }
    env.cursor = v_10;
    return true;
}
