pub static BOOLEAN: ::phf::Map<u16, &'static str> = ::phf::Map {
    key: 15467950696543387533,
    disps: &[
        (8, 33),
        (1, 17),
        (1, 0),
        (0, 30),
        (0, 18),
        (10, 7),
        (2, 8),
        (21, 19),
        (1, 2),
    ],
    entries: &[
        (39, "no_correctly_working_cr"),
        (11, "memory_above"),
        (0, "auto_left_margin"),
        (32, "has_print_wheel"),
        (5, "erase_overstrike"),
        (38, "crt_no_scrolling"),
        (15, "over_strike"),
        (8, "has_meta_key"),
        (34, "semi_auto_right_margin"),
        (18, "tilde_glitch"),
        (33, "row_addr_glitch"),
        (12, "memory_below"),
        (16, "status_line_esc_ok"),
        (28, "back_color_erase"),
        (30, "col_addr_glitch"),
        (21, "needs_xon_xoff"),
        (13, "move_insert_mode"),
        (41, "linefeed_is_newline"),
        (27, "can_change"),
        (43, "return_does_clr_eol"),
        (17, "dest_tabs_magic_smso"),
        (26, "non_dest_scroll_region"),
        (25, "no_pad_char"),
        (31, "cr_cancels_micro_mode"),
        (20, "xon_xoff"),
        (24, "non_rev_rmcup"),
        (2, "no_esc_ctlc"),
        (42, "has_hardware_tabs"),
        (29, "hue_lightness_saturation"),
        (10, "insert_null_glitch"),
        (35, "cpi_changes_res"),
        (40, "gnu_has_meta_key"),
        (22, "prtr_silent"),
        (6, "generic_type"),
        (19, "transparent_underline"),
        (36, "lpi_changes_res"),
        (14, "move_standout_mode"),
        (1, "auto_right_margin"),
        (3, "ceol_standout_glitch"),
        (9, "has_status_line"),
        (37, "backspaces_with_bs"),
        (7, "hard_copy"),
        (4, "eat_newline_glitch"),
        (23, "hard_cursor"),
    ],
};
pub static BOOLEAN_INDEX: ::phf::Map<&'static str, u16> = ::phf::Map {
    key: 7485420634051515786,
    disps: &[
        (5, 24),
        (7, 3),
        (1, 3),
        (3, 11),
        (5, 19),
        (2, 0),
        (0, 26),
        (0, 34),
        (12, 42),
    ],
    entries: &[
        ("\"tilde_glitch\"", 18),
        ("\"semi_auto_right_margin\"", 34),
        ("\"needs_xon_xoff\"", 21),
        ("\"backspaces_with_bs\"", 37),
        ("\"no_pad_char\"", 25),
        ("\"status_line_esc_ok\"", 16),
        ("\"hard_copy\"", 7),
        ("\"can_change\"", 27),
        ("\"dest_tabs_magic_smso\"", 17),
        ("\"hard_cursor\"", 23),
        ("\"move_standout_mode\"", 14),
        ("\"crt_no_scrolling\"", 38),
        ("\"memory_above\"", 11),
        ("\"no_esc_ctlc\"", 2),
        ("\"xon_xoff\"", 20),
        ("\"return_does_clr_eol\"", 43),
        ("\"col_addr_glitch\"", 30),
        ("\"over_strike\"", 15),
        ("\"gnu_has_meta_key\"", 40),
        ("\"memory_below\"", 12),
        ("\"eat_newline_glitch\"", 4),
        ("\"hue_lightness_saturation\"", 29),
        ("\"has_meta_key\"", 8),
        ("\"lpi_changes_res\"", 36),
        ("\"auto_right_margin\"", 1),
        ("\"insert_null_glitch\"", 10),
        ("\"has_print_wheel\"", 32),
        ("\"row_addr_glitch\"", 33),
        ("\"no_correctly_working_cr\"", 39),
        ("\"transparent_underline\"", 19),
        ("\"has_hardware_tabs\"", 42),
        ("\"non_dest_scroll_region\"", 26),
        ("\"cpi_changes_res\"", 35),
        ("\"cr_cancels_micro_mode\"", 31),
        ("\"ceol_standout_glitch\"", 3),
        ("\"has_status_line\"", 9),
        ("\"erase_overstrike\"", 5),
        ("\"generic_type\"", 6),
        ("\"move_insert_mode\"", 13),
        ("\"non_rev_rmcup\"", 24),
        ("\"back_color_erase\"", 28),
        ("\"prtr_silent\"", 22),
        ("\"linefeed_is_newline\"", 41),
        ("\"auto_left_margin\"", 0),
    ],
};
pub static NUMBER: ::phf::Map<u16, &'static str> = ::phf::Map {
    key: 7485420634051515786,
    disps: &[
        (0, 30),
        (9, 5),
        (12, 2),
        (2, 21),
        (0, 1),
        (12, 28),
        (24, 19),
        (2, 0),
    ],
    entries: &[
        (26, "output_res_horz_inch"),
        (8, "num_labels"),
        (21, "micro_col_size"),
        (38, "number_of_function_keys"),
        (27, "output_res_vert_inch"),
        (36, "backspace_delay"),
        (10, "label_width"),
        (23, "number_of_pins"),
        (0, "columns"),
        (7, "width_status_line"),
        (35, "new_line_delay"),
        (9, "label_height"),
        (17, "dot_vert_spacing"),
        (32, "bit_image_type"),
        (1, "init_tabs"),
        (18, "dot_horz_spacing"),
        (5, "padding_baud_rate"),
        (20, "max_micro_jump"),
        (25, "output_res_line"),
        (12, "maximum_windows"),
        (2, "lines"),
        (4, "magic_cookie_glitch"),
        (14, "max_pairs"),
        (15, "no_color_video"),
        (33, "magic_cookie_glitch_ul"),
        (31, "bit_image_entwining"),
        (24, "output_res_char"),
        (30, "buttons"),
        (3, "lines_of_memory"),
        (11, "max_attributes"),
        (34, "carriage_return_delay"),
        (22, "micro_line_size"),
        (19, "max_micro_address"),
        (16, "buffer_capacity"),
        (6, "virtual_terminal"),
        (29, "wide_char_size"),
        (28, "print_rate"),
        (13, "max_colors"),
        (37, "horizontal_tab_delay"),
    ],
};
pub static NUMBER_INDEX: ::phf::Map<&'static str, u16> = ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
        (14, 3),
        (2, 5),
        (2, 37),
        (0, 8),
        (3, 6),
        (9, 21),
        (3, 33),
    ],
    entries: &[
        ("\"num_labels\"", 8),
        ("\"output_res_char\"", 24),
        ("\"new_line_delay\"", 35),
        ("\"carriage_return_delay\"", 34),
        ("\"maximum_windows\"", 12),
        ("\"max_pairs\"", 14),
        ("\"print_rate\"", 28),
        ("\"max_colors\"", 13),
        ("\"columns\"", 0),
        ("\"backspace_delay\"", 36),
        ("\"no_color_video\"", 15),
        ("\"virtual_terminal\"", 6),
        ("\"lines\"", 2),
        ("\"max_micro_jump\"", 20),
        ("\"output_res_vert_inch\"", 27),
        ("\"micro_col_size\"", 21),
        ("\"padding_baud_rate\"", 5),
        ("\"bit_image_type\"", 32),
        ("\"dot_vert_spacing\"", 17),
        ("\"label_height\"", 9),
        ("\"magic_cookie_glitch\"", 4),
        ("\"wide_char_size\"", 29),
        ("\"magic_cookie_glitch_ul\"", 33),
        ("\"number_of_pins\"", 23),
        ("\"max_attributes\"", 11),
        ("\"lines_of_memory\"", 3),
        ("\"micro_line_size\"", 22),
        ("\"dot_horz_spacing\"", 18),
        ("\"buffer_capacity\"", 16),
        ("\"width_status_line\"", 7),
        ("\"init_tabs\"", 1),
        ("\"label_width\"", 10),
        ("\"output_res_line\"", 25),
        ("\"buttons\"", 30),
        ("\"bit_image_entwining\"", 31),
        ("\"number_of_function_keys\"", 38),
        ("\"max_micro_address\"", 19),
        ("\"output_res_horz_inch\"", 26),
        ("\"horizontal_tab_delay\"", 37),
    ],
};
pub static STRING: ::phf::Map<u16, &'static str> = ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 10),
        (0, 412),
        (0, 220),
        (0, 66),
        (0, 38),
        (0, 241),
        (0, 0),
        (0, 13),
        (0, 0),
        (0, 305),
        (0, 73),
        (0, 84),
        (2, 270),
        (0, 37),
        (0, 72),
        (0, 50),
        (0, 0),
        (0, 189),
        (0, 34),
        (0, 2),
        (0, 188),
        (0, 4),
        (1, 380),
        (0, 312),
        (0, 247),
        (2, 137),
        (4, 71),
        (0, 232),
        (0, 7),
        (0, 44),
        (0, 1),
        (0, 150),
        (0, 25),
        (0, 249),
        (0, 7),
        (0, 44),
        (0, 328),
        (0, 6),
        (0, 339),
        (0, 21),
        (0, 5),
        (0, 0),
        (0, 36),
        (0, 4),
        (0, 234),
        (0, 1),
        (6, 299),
        (0, 0),
        (0, 6),
        (0, 7),
        (2, 330),
        (1, 181),
        (0, 210),
        (0, 0),
        (0, 123),
        (0, 7),
        (4, 127),
        (3, 67),
        (14, 129),
        (3, 293),
        (15, 34),
        (0, 0),
        (0, 16),
        (0, 326),
        (2, 84),
        (0, 412),
        (2, 212),
        (11, 318),
        (2, 190),
        (19, 393),
        (4, 218),
        (5, 363),
        (1, 344),
        (0, 12),
        (0, 301),
        (2, 11),
        (27, 41),
        (0, 2),
        (0, 83),
        (0, 12),
        (7, 150),
        (0, 6),
        (0, 92),
    ],
    entries: &[
        (47, "from_status_line"),
        (234, "key_f29"),
        (109, "parm_index"),
        (92, "lab_f10"),
        (103, "newline"),
        (179, "key_refresh"),
        (98, "lab_f7"),
        (68, "key_f2"),
        (95, "lab_f4"),
        (29, "enter_delete_mode"),
        (259, "key_f54"),
        (9, "command_character"),
        (186, "key_sbeg"),
        (396, "linefeed_if_not_lf"),
        (214, "key_sundo"),
        (11, "cursor_down"),
        (32, "enter_secure_mode"),
        (279, "hangup"),
        (361, "pkey_plab"),
        (288, "user1"),
        (108, "parm_ich"),
        (320, "exit_doublewide_mode"),
        (403, "acs_lrcorner"),
        (245, "key_f40"),
        (76, "key_home"),
        (138, "init_prog"),
        (165, "key_enter"),
        (265, "key_f60"),
        (240, "key_f35"),
        (364, "set0_des_seq"),
        (405, "acs_rtee"),
        (208, "key_sredo"),
        (15, "cursor_mem_address"),
        (53, "insert_line"),
        (46, "form_feed"),
        (21, "delete_character"),
        (266, "key_f61"),
        (195, "key_seol"),
        (321, "exit_italics_mode"),
        (180, "key_replace"),
        (146, "acs_chars"),
        (400, "acs_ulcorner"),
        (224, "key_f19"),
        (262, "key_f57"),
        (385, "alt_scancode_esc"),
        (38, "exit_alt_charset_mode"),
        (225, "key_f20"),
        (278, "goto_window"),
        (203, "key_smove"),
        (392, "set_a_attributes"),
        (329, "micro_down"),
        (132, "set_tab"),
        (217, "key_f12"),
        (36, "enter_underline_mode"),
        (339, "select_char_set"),
        (150, "exit_xon_mode"),
        (204, "key_snext"),
        (90, "lab_f0"),
        (295, "user8"),
        (65, "key_f0"),
        (267, "key_f62"),
        (366, "set2_des_seq"),
        (227, "key_f22"),
        (286, "wait_tone"),
        (274, "set_clock"),
        (196, "key_sexit"),
        (296, "user9"),
        (397, "backspace_if_not_bs"),
        (35, "enter_standout_mode"),
        (162, "key_copy"),
        (16, "cursor_normal"),
        (166, "key_exit"),
        (390, "enter_top_hl_mode"),
        (104, "pad_char"),
        (202, "key_smessage"),
        (115, "pkey_key"),
        (134, "tab"),
        (167, "key_find"),
        (158, "key_beg"),
        (145, "char_padding"),
        (384, "scancode_escape"),
        (254, "key_f49"),
        (74, "key_f8"),
        (268, "key_f63"),
        (192, "key_sdl"),
        (271, "set_left_margin"),
        (210, "key_sright"),
        (27, "enter_bold_mode"),
        (10, "cursor_address"),
        (299, "initialize_color"),
        (328, "micro_column_address"),
        (75, "key_f9"),
        (129, "scroll_forward"),
        (330, "micro_left"),
        (255, "key_f50"),
        (82, "key_ppage"),
        (43, "exit_standout_mode"),
        (261, "key_f56"),
        (407, "acs_ttee"),
        (249, "key_f44"),
        (182, "key_resume"),
        (246, "key_f41"),
        (66, "key_f1"),
        (324, "exit_shadow_mode"),
        (401, "acs_llcorner"),
        (200, "key_sic"),
        (14, "cursor_left"),
        (347, "start_char_set_def"),
        (336, "parm_left_micro"),
        (122, "reset_1string"),
        (270, "clear_margins"),
        (360, "set_a_background"),
        (42, "exit_insert_mode"),
        (144, "prtr_non"),
        (84, "key_sf"),
        (380, "exit_pc_charset_mode"),
        (174, "key_options"),
        (273, "label_format"),
        (350, "subscript_characters"),
        (96, "lab_f5"),
        (6, "clr_eol"),
        (322, "exit_leftward_mode"),
        (123, "reset_2string"),
        (154, "xoff_character"),
        (88, "keypad_local"),
        (248, "key_f43"),
        (398, "other_non_function_keys"),
        (121, "repeat_char"),
        (243, "key_f38"),
        (169, "key_mark"),
        (264, "key_f59"),
        (257, "key_f52"),
        (23, "dis_status_line"),
        (173, "key_open"),
        (130, "scroll_reverse"),
        (305, "change_line_pitch"),
        (343, "set_right_margin_parm"),
        (102, "meta_on"),
        (153, "xon_character"),
        (193, "key_select"),
        (194, "key_send"),
        (365, "set1_des_seq"),
        (342, "set_left_margin_parm"),
        (231, "key_f26"),
        (87, "key_up"),
        (344, "set_top_margin"),
        (28, "enter_ca_mode"),
        (260, "key_f55"),
        (381, "enter_scancode_mode"),
        (124, "reset_3string"),
        (340, "set_bottom_margin"),
        (51, "init_file"),
        (213, "key_ssuspend"),
        (280, "dial_phone"),
        (378, "display_pc_char"),
        (298, "orig_colors"),
        (318, "enter_superscript_mode"),
        (315, "enter_normal_quality"),
        (275, "display_clock"),
        (389, "enter_right_hl_mode"),
        (372, "bit_image_carriage_return"),
        (387, "enter_left_hl_mode"),
        (177, "key_redo"),
        (40, "exit_ca_mode"),
        (67, "key_f10"),
        (101, "meta_off"),
        (52, "insert_character"),
        (335, "parm_down_micro"),
        (402, "acs_urcorner"),
        (352, "these_cause_cr"),
        (317, "enter_subscript_mode"),
        (351, "superscript_characters"),
        (1, "bell"),
        (91, "lab_f1"),
        (44, "exit_underline_mode"),
        (116, "pkey_local"),
        (70, "key_f4"),
        (370, "bit_image_repeat"),
        (31, "enter_insert_mode"),
        (221, "key_f16"),
        (292, "user5"),
        (386, "enter_horizontal_hl_mode"),
        (140, "key_a3"),
        (17, "cursor_right"),
        (363, "code_set_init"),
        (19, "cursor_up"),
        (171, "key_move"),
        (412, "memory_unlock"),
        (120, "prtr_on"),
        (69, "key_f3"),
        (223, "key_f18"),
        (77, "key_ic"),
        (94, "lab_f3"),
        (127, "row_address"),
        (356, "mouse_info"),
        (393, "set_pglen_inch"),
        (56, "key_catab"),
        (112, "parm_right_cursor"),
        (83, "key_right"),
        (291, "user4"),
        (313, "enter_micro_mode"),
        (126, "restore_cursor"),
        (311, "enter_italics_mode"),
        (302, "set_foreground"),
        (294, "user7"),
        (45, "flash_screen"),
        (409, "acs_vline"),
        (172, "key_next"),
        (222, "key_f17"),
        (314, "enter_near_letter_quality"),
        (201, "key_sleft"),
        (354, "char_set_names"),
        (22, "delete_line"),
        (79, "key_left"),
        (353, "zero_motion"),
        (413, "box_chars_1"),
        (55, "key_backspace"),
        (272, "set_right_margin"),
        (34, "enter_reverse_mode"),
        (188, "key_scommand"),
        (170, "key_message"),
        (197, "key_sfind"),
        (357, "req_mouse_pos"),
        (161, "key_command"),
        (236, "key_f31"),
        (358, "get_mouse"),
        (410, "acs_plus"),
        (128, "save_cursor"),
        (12, "cursor_home"),
        (326, "exit_superscript_mode"),
        (395, "termcap_reset"),
        (151, "enter_am_mode"),
        (283, "pulse"),
        (178, "key_reference"),
        (276, "remove_clock"),
        (238, "key_f33"),
        (368, "set_lr_margin"),
        (373, "color_names"),
        (250, "key_f45"),
        (73, "key_f7"),
        (258, "key_f53"),
        (300, "initialize_pair"),
        (206, "key_sprevious"),
        (93, "lab_f2"),
        (86, "key_stab"),
        (125, "reset_file"),
        (215, "req_for_input"),
        (237, "key_f32"),
        (147, "plab_norm"),
        (293, "user6"),
        (114, "parm_up_cursor"),
        (216, "key_f11"),
        (18, "cursor_to_ll"),
        (30, "enter_dim_mode"),
        (232, "key_f27"),
        (149, "enter_xon_mode"),
        (332, "micro_row_address"),
        (71, "key_f5"),
        (277, "create_window"),
        (327, "exit_upward_mode"),
        (26, "enter_blink_mode"),
        (4, "clear_all_tabs"),
        (183, "key_save"),
        (2, "carriage_return"),
        (341, "set_bottom_margin_parm"),
        (117, "pkey_xmit"),
        (303, "set_background"),
        (50, "init_3string"),
        (346, "start_bit_image"),
        (308, "define_char"),
        (399, "arrow_key_map"),
        (106, "parm_delete_line"),
        (20, "cursor_visible"),
        (218, "key_f13"),
        (89, "keypad_xmit"),
        (60, "key_dl"),
        (349, "stop_char_set_def"),
        (355, "key_mouse"),
        (290, "user3"),
        (394, "termcap_init2"),
        (176, "key_print"),
        (54, "insert_padding"),
        (59, "key_dc"),
        (284, "flash_hook"),
        (253, "key_f48"),
        (359, "set_a_foreground"),
        (156, "label_on"),
        (163, "key_create"),
        (185, "key_undo"),
        (331, "micro_right"),
        (247, "key_f42"),
        (7, "clr_eos"),
        (143, "key_c3"),
        (64, "key_eos"),
        (157, "label_off"),
        (81, "key_npage"),
        (110, "parm_insert_line"),
        (37, "erase_chars"),
        (345, "set_top_margin_parm"),
        (219, "key_f14"),
        (58, "key_ctab"),
        (325, "exit_subscript_mode"),
        (118, "print_screen"),
        (160, "key_close"),
        (374, "define_bit_image_region"),
        (282, "tone"),
        (48, "init_1string"),
        (0, "back_tab"),
        (338, "parm_up_micro"),
        (369, "set_tb_margin"),
        (187, "key_scancel"),
        (5, "clear_screen"),
        (61, "key_down"),
        (404, "acs_ltee"),
        (242, "key_f37"),
        (233, "key_f28"),
        (411, "memory_lock"),
        (181, "key_restart"),
        (244, "key_f39"),
        (377, "set_page_length"),
        (25, "enter_alt_charset_mode"),
        (13, "cursor_invisible"),
        (148, "key_btab"),
        (312, "enter_leftward_mode"),
        (304, "change_char_pitch"),
        (113, "parm_rindex"),
        (49, "init_2string"),
        (334, "order_of_pins"),
        (301, "set_color_pair"),
        (285, "fixed_pause"),
        (235, "key_f30"),
        (133, "set_window"),
        (375, "end_bit_image_region"),
        (119, "prtr_off"),
        (333, "micro_up"),
        (323, "exit_micro_mode"),
        (228, "key_f23"),
        (388, "enter_low_hl_mode"),
        (152, "exit_am_mode"),
        (57, "key_clear"),
        (100, "lab_f9"),
        (190, "key_screate"),
        (136, "underline_char"),
        (226, "key_f21"),
        (41, "exit_delete_mode"),
        (367, "set3_des_seq"),
        (72, "key_f6"),
        (211, "key_srsume"),
        (137, "up_half_line"),
        (39, "exit_attribute_mode"),
        (406, "acs_btee"),
        (362, "device_type"),
        (371, "bit_image_newline"),
        (251, "key_f46"),
        (316, "enter_shadow_mode"),
        (310, "enter_draft_quality"),
        (199, "key_shome"),
        (382, "exit_scancode_mode"),
        (184, "key_suspend"),
        (155, "ena_acs"),
        (205, "key_soptions"),
        (164, "key_end"),
        (142, "key_c1"),
        (229, "key_f24"),
        (319, "enter_upward_mode"),
        (252, "key_f47"),
        (105, "parm_dch"),
        (289, "user2"),
        (239, "key_f34"),
        (256, "key_f51"),
        (269, "clr_bol"),
        (297, "orig_pair"),
        (337, "parm_right_micro"),
        (306, "change_res_horz"),
        (379, "enter_pc_charset_mode"),
        (111, "parm_left_cursor"),
        (78, "key_il"),
        (309, "enter_doublewide_mode"),
        (159, "key_cancel"),
        (287, "user0"),
        (383, "pc_term_options"),
        (62, "key_eic"),
        (307, "change_res_vert"),
        (198, "key_shelp"),
        (8, "column_address"),
        (230, "key_f25"),
        (168, "key_help"),
        (107, "parm_down_cursor"),
        (391, "enter_vertical_hl_mode"),
        (80, "key_ll"),
        (212, "key_ssave"),
        (135, "to_status_line"),
        (281, "quick_dial"),
        (63, "key_eol"),
        (209, "key_sreplace"),
        (348, "stop_bit_image"),
        (263, "key_f58"),
        (207, "key_sprint"),
        (408, "acs_hline"),
        (191, "key_sdc"),
        (85, "key_sr"),
        (175, "key_previous"),
        (33, "enter_protected_mode"),
        (139, "key_a1"),
        (241, "key_f36"),
        (97, "lab_f6"),
        (99, "lab_f8"),
        (24, "down_half_line"),
        (131, "set_attributes"),
        (376, "set_color_band"),
        (220, "key_f15"),
        (141, "key_b2"),
        (189, "key_scopy"),
        (3, "change_scroll_region"),
    ],
};
pub static STRING_INDEX: ::phf::Map<&'static str, u16> = ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 18),
        (0, 0),
        (0, 1),
        (2, 55),
        (0, 44),
        (0, 100),
        (0, 0),
        (0, 0),
        (1, 0),
        (0, 353),
        (0, 11),
        (0, 56),
        (0, 0),
        (0, 14),
        (0, 40),
        (1, 5),
        (0, 51),
        (0, 90),
        (0, 0),
        (0, 46),
        (0, 0),
        (0, 2),
        (0, 226),
        (0, 18),
        (1, 155),
        (0, 22),
        (0, 66),
        (3, 176),
        (0, 64),
        (0, 209),
        (0, 328),
        (0, 295),
        (0, 38),
        (0, 260),
        (0, 22),
        (1, 11),
        (1, 207),
        (0, 318),
        (0, 53),
        (0, 10),
        (6, 289),
        (0, 4),
        (0, 18),
        (0, 263),
        (0, 2),
        (0, 12),
        (0, 0),
        (1, 37),
        (0, 41),
        (7, 164),
        (0, 1),
        (0, 8),
        (0, 10),
        (3, 259),
        (1, 133),
        (0, 10),
        (3, 366),
        (0, 228),
        (0, 0),
        (0, 215),
        (0, 2),
        (3, 0),
        (0, 247),
        (17, 241),
        (4, 6),
        (0, 2),
        (8, 357),
        (2, 148),
        (17, 177),
        (0, 0),
        (7, 275),
        (2, 250),
        (0, 1),
        (0, 35),
        (0, 96),
        (3, 102),
        (2, 9),
        (0, 5),
        (13, 181),
        (0, 235),
        (0, 348),
        (1, 160),
        (0, 25),
    ],
    entries: &[
        ("\"start_bit_image\"", 346),
        ("\"set_a_attributes\"", 392),
        ("\"key_f28\"", 233),
        ("\"key_dl\"", 60),
        ("\"key_move\"", 171),
        ("\"key_f40\"", 245),
        ("\"key_send\"", 194),
        ("\"set_top_margin_parm\"", 345),
        ("\"key_f33\"", 238),
        ("\"key_shome\"", 199),
        ("\"acs_ulcorner\"", 400),
        ("\"micro_up\"", 333),
        ("\"plab_norm\"", 147),
        ("\"parm_right_cursor\"", 112),
        ("\"enter_xon_mode\"", 149),
        ("\"key_next\"", 172),
        ("\"key_sbeg\"", 186),
        ("\"key_ssave\"", 212),
        ("\"enter_insert_mode\"", 31),
        ("\"key_f27\"", 232),
        ("\"enter_leftward_mode\"", 312),
        ("\"key_ppage\"", 82),
        ("\"key_restart\"", 181),
        ("\"exit_delete_mode\"", 41),
        ("\"acs_urcorner\"", 402),
        ("\"key_sright\"", 210),
        ("\"orig_colors\"", 298),
        ("\"prtr_off\"", 119),
        ("\"user7\"", 294),
        ("\"key_ic\"", 77),
        ("\"stop_char_set_def\"", 349),
        ("\"key_f9\"", 75),
        ("\"flash_screen\"", 45),
        ("\"key_smessage\"", 202),
        ("\"key_end\"", 164),
        ("\"key_a1\"", 139),
        ("\"micro_column_address\"", 328),
        ("\"key_down\"", 61),
        ("\"cursor_right\"", 17),
        ("\"key_home\"", 76),
        ("\"key_sredo\"", 208),
        ("\"cursor_visible\"", 20),
        ("\"acs_llcorner\"", 401),
        ("\"key_scancel\"", 187),
        ("\"micro_left\"", 330),
        ("\"key_f56\"", 261),
        ("\"key_cancel\"", 159),
        ("\"key_f16\"", 221),
        ("\"key_f18\"", 223),
        ("\"exit_attribute_mode\"", 39),
        ("\"key_backspace\"", 55),
        ("\"key_f63\"", 268),
        ("\"enter_alt_charset_mode\"", 25),
        ("\"fixed_pause\"", 285),
        ("\"key_sprevious\"", 206),
        ("\"mouse_info\"", 356),
        ("\"up_half_line\"", 137),
        ("\"cursor_to_ll\"", 18),
        ("\"these_cause_cr\"", 352),
        ("\"user0\"", 287),
        ("\"enter_underline_mode\"", 36),
        ("\"key_f13\"", 218),
        ("\"create_window\"", 277),
        ("\"lab_f9\"", 100),
        ("\"key_f3\"", 69),
        ("\"enter_draft_quality\"", 310),
        ("\"xon_character\"", 153),
        ("\"key_f60\"", 265),
        ("\"bit_image_repeat\"", 370),
        ("\"exit_standout_mode\"", 43),
        ("\"key_eic\"", 62),
        ("\"enter_blink_mode\"", 26),
        ("\"user5\"", 292),
        ("\"enter_pc_charset_mode\"", 379),
        ("\"key_f20\"", 225),
        ("\"exit_pc_charset_mode\"", 380),
        ("\"hangup\"", 279),
        ("\"key_refresh\"", 179),
        ("\"init_prog\"", 138),
        ("\"key_beg\"", 158),
        ("\"enter_am_mode\"", 151),
        ("\"cursor_address\"", 10),
        ("\"key_soptions\"", 205),
        ("\"key_smove\"", 203),
        ("\"key_help\"", 168),
        ("\"acs_rtee\"", 405),
        ("\"wait_tone\"", 286),
        ("\"back_tab\"", 0),
        ("\"other_non_function_keys\"", 398),
        ("\"key_command\"", 161),
        ("\"key_f51\"", 256),
        ("\"enter_italics_mode\"", 311),
        ("\"set_background\"", 303),
        ("\"key_options\"", 174),
        ("\"key_redo\"", 177),
        ("\"delete_character\"", 21),
        ("\"stop_bit_image\"", 348),
        ("\"exit_am_mode\"", 152),
        ("\"linefeed_if_not_lf\"", 396),
        ("\"pkey_xmit\"", 117),
        ("\"prtr_on\"", 120),
        ("\"exit_superscript_mode\"", 326),
        ("\"key_f4\"", 70),
        ("\"lab_f8\"", 99),
        ("\"key_f48\"", 253),
        ("\"key_close\"", 160),
        ("\"key_btab\"", 148),
        ("\"zero_motion\"", 353),
        ("\"enter_vertical_hl_mode\"", 391),
        ("\"set_bottom_margin_parm\"", 341),
        ("\"parm_left_micro\"", 336),
        ("\"key_sfind\"", 197),
        ("\"user9\"", 296),
        ("\"key_f5\"", 71),
        ("\"key_ctab\"", 58),
        ("\"pc_term_options\"", 383),
        ("\"key_f32\"", 237),
        ("\"lab_f5\"", 96),
        ("\"parm_ich\"", 108),
        ("\"acs_btee\"", 406),
        ("\"set_page_length\"", 377),
        ("\"parm_rindex\"", 113),
        ("\"enter_standout_mode\"", 35),
        ("\"insert_padding\"", 54),
        ("\"scancode_escape\"", 384),
        ("\"print_screen\"", 118),
        ("\"key_f53\"", 258),
        ("\"set_lr_margin\"", 368),
        ("\"key_left\"", 79),
        ("\"box_chars_1\"", 413),
        ("\"key_f44\"", 249),
        ("\"set2_des_seq\"", 366),
        ("\"dial_phone\"", 280),
        ("\"parm_up_cursor\"", 114),
        ("\"lab_f0\"", 90),
        ("\"exit_underline_mode\"", 44),
        ("\"acs_ltee\"", 404),
        ("\"cursor_up\"", 19),
        ("\"reset_2string\"", 123),
        ("\"bit_image_newline\"", 371),
        ("\"key_f17\"", 222),
        ("\"set_window\"", 133),
        ("\"lab_f1\"", 91),
        ("\"key_select\"", 193),
        ("\"key_b2\"", 141),
        ("\"enter_top_hl_mode\"", 390),
        ("\"meta_on\"", 102),
        ("\"acs_chars\"", 146),
        ("\"key_enter\"", 165),
        ("\"enter_right_hl_mode\"", 389),
        ("\"newline\"", 103),
        ("\"ena_acs\"", 155),
        ("\"enter_normal_quality\"", 315),
        ("\"key_reference\"", 178),
        ("\"label_on\"", 156),
        ("\"key_mouse\"", 355),
        ("\"pad_char\"", 104),
        ("\"parm_down_cursor\"", 107),
        ("\"get_mouse\"", 358),
        ("\"key_f50\"", 255),
        ("\"key_sic\"", 200),
        ("\"key_f30\"", 235),
        ("\"key_catab\"", 56),
        ("\"key_screate\"", 190),
        ("\"set_color_band\"", 376),
        ("\"key_f45\"", 250),
        ("\"init_2string\"", 49),
        ("\"memory_lock\"", 411),
        ("\"key_f0\"", 65),
        ("\"enter_delete_mode\"", 29),
        ("\"restore_cursor\"", 126),
        ("\"label_off\"", 157),
        ("\"key_dc\"", 59),
        ("\"reset_file\"", 125),
        ("\"initialize_pair\"", 300),
        ("\"repeat_char\"", 121),
        ("\"key_f8\"", 74),
        ("\"label_format\"", 273),
        ("\"key_f54\"", 259),
        ("\"key_f15\"", 220),
        ("\"key_sf\"", 84),
        ("\"user3\"", 290),
        ("\"set_clock\"", 274),
        ("\"exit_scancode_mode\"", 382),
        ("\"lab_f3\"", 94),
        ("\"key_f46\"", 251),
        ("\"order_of_pins\"", 334),
        ("\"enter_shadow_mode\"", 316),
        ("\"key_sexit\"", 196),
        ("\"acs_hline\"", 408),
        ("\"quick_dial\"", 281),
        ("\"termcap_init2\"", 394),
        ("\"memory_unlock\"", 412),
        ("\"remove_clock\"", 276),
        ("\"key_f38\"", 243),
        ("\"key_f42\"", 247),
        ("\"key_f59\"", 264),
        ("\"key_f57\"", 262),
        ("\"scroll_reverse\"", 130),
        ("\"acs_lrcorner\"", 403),
        ("\"key_copy\"", 162),
        ("\"key_stab\"", 86),
        ("\"change_res_vert\"", 307),
        ("\"key_resume\"", 182),
        ("\"key_a3\"", 140),
        ("\"user1\"", 288),
        ("\"cursor_invisible\"", 13),
        ("\"user2\"", 289),
        ("\"key_sreplace\"", 209),
        ("\"init_3string\"", 50),
        ("\"micro_down\"", 329),
        ("\"exit_ca_mode\"", 40),
        ("\"key_sundo\"", 214),
        ("\"enter_protected_mode\"", 33),
        ("\"exit_alt_charset_mode\"", 38),
        ("\"key_up\"", 87),
        ("\"key_right\"", 83),
        ("\"key_clear\"", 57),
        ("\"set_right_margin_parm\"", 343),
        ("\"dis_status_line\"", 23),
        ("\"color_names\"", 373),
        ("\"key_ssuspend\"", 213),
        ("\"display_clock\"", 275),
        ("\"key_scommand\"", 188),
        ("\"backspace_if_not_bs\"", 397),
        ("\"exit_leftward_mode\"", 322),
        ("\"clr_eol\"", 6),
        ("\"key_previous\"", 175),
        ("\"key_print\"", 176),
        ("\"cursor_normal\"", 16),
        ("\"key_open\"", 173),
        ("\"bit_image_carriage_return\"", 372),
        ("\"set1_des_seq\"", 365),
        ("\"parm_delete_line\"", 106),
        ("\"set_top_margin\"", 344),
        ("\"key_f25\"", 230),
        ("\"lab_f10\"", 92),
        ("\"code_set_init\"", 363),
        ("\"start_char_set_def\"", 347),
        ("\"user8\"", 295),
        ("\"clr_bol\"", 269),
        ("\"scroll_forward\"", 129),
        ("\"device_type\"", 362),
        ("\"set3_des_seq\"", 367),
        ("\"key_f22\"", 227),
        ("\"exit_upward_mode\"", 327),
        ("\"cursor_home\"", 12),
        ("\"enter_bold_mode\"", 27),
        ("\"user6\"", 293),
        ("\"clr_eos\"", 7),
        ("\"req_for_input\"", 215),
        ("\"char_padding\"", 145),
        ("\"key_f2\"", 68),
        ("\"prtr_non\"", 144),
        ("\"underline_char\"", 136),
        ("\"set_foreground\"", 302),
        ("\"key_f58\"", 263),
        ("\"key_f24\"", 229),
        ("\"key_undo\"", 185),
        ("\"exit_insert_mode\"", 42),
        ("\"meta_off\"", 101),
        ("\"set_left_margin_parm\"", 342),
        ("\"enter_ca_mode\"", 28),
        ("\"key_scopy\"", 189),
        ("\"key_ll\"", 80),
        ("\"key_f23\"", 228),
        ("\"enter_subscript_mode\"", 317),
        ("\"clear_all_tabs\"", 4),
        ("\"enter_superscript_mode\"", 318),
        ("\"parm_up_micro\"", 338),
        ("\"reset_1string\"", 122),
        ("\"define_char\"", 308),
        ("\"keypad_xmit\"", 89),
        ("\"key_f62\"", 267),
        ("\"initialize_color\"", 299),
        ("\"key_c3\"", 143),
        ("\"req_mouse_pos\"", 357),
        ("\"key_sdl\"", 192),
        ("\"command_character\"", 9),
        ("\"clear_screen\"", 5),
        ("\"key_f55\"", 260),
        ("\"set_a_background\"", 360),
        ("\"key_create\"", 163),
        ("\"change_line_pitch\"", 305),
        ("\"insert_line\"", 53),
        ("\"key_f43\"", 248),
        ("\"set_tb_margin\"", 369),
        ("\"parm_down_micro\"", 335),
        ("\"pkey_plab\"", 361),
        ("\"change_res_horz\"", 306),
        ("\"enter_horizontal_hl_mode\"", 386),
        ("\"key_sleft\"", 201),
        ("\"parm_index\"", 109),
        ("\"key_f10\"", 67),
        ("\"key_c1\"", 142),
        ("\"key_srsume\"", 211),
        ("\"lab_f7\"", 98),
        ("\"parm_dch\"", 105),
        ("\"xoff_character\"", 154),
        ("\"enter_near_letter_quality\"", 314),
        ("\"set_left_margin\"", 271),
        ("\"enter_low_hl_mode\"", 388),
        ("\"micro_right\"", 331),
        ("\"parm_right_micro\"", 337),
        ("\"key_f41\"", 246),
        ("\"enter_scancode_mode\"", 381),
        ("\"column_address\"", 8),
        ("\"cursor_mem_address\"", 15),
        ("\"subscript_characters\"", 350),
        ("\"acs_ttee\"", 407),
        ("\"key_f7\"", 73),
        ("\"key_f34\"", 239),
        ("\"bell\"", 1),
        ("\"enter_left_hl_mode\"", 387),
        ("\"row_address\"", 127),
        ("\"exit_shadow_mode\"", 324),
        ("\"arrow_key_map\"", 399),
        ("\"key_message\"", 170),
        ("\"key_il\"", 78),
        ("\"end_bit_image_region\"", 375),
        ("\"key_npage\"", 81),
        ("\"enter_reverse_mode\"", 34),
        ("\"exit_xon_mode\"", 150),
        ("\"exit_subscript_mode\"", 325),
        ("\"pkey_local\"", 116),
        ("\"tab\"", 134),
        ("\"init_1string\"", 48),
        ("\"key_sr\"", 85),
        ("\"key_f39\"", 244),
        ("\"key_f14\"", 219),
        ("\"key_sprint\"", 207),
        ("\"enter_upward_mode\"", 319),
        ("\"tone\"", 282),
        ("\"key_sdc\"", 191),
        ("\"key_f35\"", 240),
        ("\"key_f31\"", 236),
        ("\"key_eol\"", 63),
        ("\"char_set_names\"", 354),
        ("\"save_cursor\"", 128),
        ("\"to_status_line\"", 135),
        ("\"key_seol\"", 195),
        ("\"set_color_pair\"", 301),
        ("\"insert_character\"", 52),
        ("\"display_pc_char\"", 378),
        ("\"key_f12\"", 217),
        ("\"key_f26\"", 231),
        ("\"set_tab\"", 132),
        ("\"erase_chars\"", 37),
        ("\"define_bit_image_region\"", 374),
        ("\"key_exit\"", 166),
        ("\"exit_italics_mode\"", 321),
        ("\"lab_f4\"", 95),
        ("\"enter_doublewide_mode\"", 309),
        ("\"superscript_characters\"", 351),
        ("\"set_a_foreground\"", 359),
        ("\"form_feed\"", 46),
        ("\"key_f36\"", 241),
        ("\"set_bottom_margin\"", 340),
        ("\"key_f19\"", 224),
        ("\"micro_row_address\"", 332),
        ("\"reset_3string\"", 124),
        ("\"keypad_local\"", 88),
        ("\"cursor_down\"", 11),
        ("\"change_char_pitch\"", 304),
        ("\"key_f21\"", 226),
        ("\"enter_dim_mode\"", 30),
        ("\"pkey_key\"", 115),
        ("\"delete_line\"", 22),
        ("\"key_snext\"", 204),
        ("\"lab_f2\"", 93),
        ("\"orig_pair\"", 297),
        ("\"init_file\"", 51),
        ("\"key_f11\"", 216),
        ("\"change_scroll_region\"", 3),
        ("\"key_f49\"", 254),
        ("\"parm_left_cursor\"", 111),
        ("\"exit_doublewide_mode\"", 320),
        ("\"user4\"", 291),
        ("\"parm_insert_line\"", 110),
        ("\"exit_micro_mode\"", 323),
        ("\"key_f61\"", 266),
        ("\"key_f47\"", 252),
        ("\"cursor_left\"", 14),
        ("\"key_find\"", 167),
        ("\"acs_plus\"", 410),
        ("\"key_replace\"", 180),
        ("\"flash_hook\"", 284),
        ("\"down_half_line\"", 24),
        ("\"enter_secure_mode\"", 32),
        ("\"set0_des_seq\"", 364),
        ("\"key_f1\"", 66),
        ("\"set_attributes\"", 131),
        ("\"carriage_return\"", 2),
        ("\"key_eos\"", 64),
        ("\"termcap_reset\"", 395),
        ("\"key_f29\"", 234),
        ("\"select_char_set\"", 339),
        ("\"lab_f6\"", 97),
        ("\"key_mark\"", 169),
        ("\"clear_margins\"", 270),
        ("\"alt_scancode_esc\"", 385),
        ("\"key_f52\"", 257),
        ("\"key_save\"", 183),
        ("\"from_status_line\"", 47),
        ("\"enter_micro_mode\"", 313),
        ("\"key_f37\"", 242),
        ("\"key_f6\"", 72),
        ("\"goto_window\"", 278),
        ("\"acs_vline\"", 409),
        ("\"set_pglen_inch\"", 393),
        ("\"set_right_margin\"", 272),
        ("\"pulse\"", 283),
        ("\"key_suspend\"", 184),
        ("\"key_shelp\"", 198),
    ],
};
pub static TERMINFO: ::phf::Map<&'static str, &'static str> = ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 202),
        (0, 379),
        (0, 113),
        (0, 236),
        (0, 124),
        (0, 6),
        (0, 0),
        (0, 20),
        (1, 2),
        (0, 1),
        (0, 0),
        (0, 204),
        (0, 5),
        (61, 258),
        (0, 109),
        (7, 152),
        (1, 338),
        (1, 132),
        (0, 3),
        (0, 223),
        (1, 54),
        (0, 16),
        (0, 49),
        (0, 59),
        (0, 49),
        (1, 4),
        (1, 403),
        (0, 0),
        (0, 0),
        (0, 10),
        (0, 115),
        (0, 49),
        (0, 1),
        (0, 1),
        (0, 19),
        (0, 39),
        (0, 11),
        (0, 38),
        (0, 23),
        (0, 57),
        (0, 12),
        (0, 58),
        (0, 22),
        (0, 53),
        (0, 1),
        (0, 209),
        (2, 49),
        (0, 71),
        (0, 298),
        (3, 300),
        (0, 22),
        (0, 0),
        (0, 60),
        (0, 0),
        (2, 322),
        (0, 2),
        (0, 41),
        (0, 350),
        (2, 23),
        (0, 0),
        (2, 130),
        (4, 239),
        (1, 198),
        (2, 76),
        (24, 54),
        (11, 297),
        (0, 58),
        (19, 245),
        (9, 306),
        (153, 164),
        (1, 154),
        (1, 7),
        (1, 342),
        (0, 6),
        (7, 403),
        (2, 337),
        (46, 104),
        (0, 198),
        (0, 22),
        (0, 121),
        (0, 0),
    ],
    entries: &[
        ("key_redo", "krdo"),
        ("key_scopy", "kCPY"),
        ("prtr_on", "mc5"),
        ("key_message", "kmsg"),
        ("ena_acs", "enacs"),
        ("save_cursor", "sc"),
        ("print_screen", "mc0"),
        ("max_micro_address", "maddr"),
        ("init_2string", "is2"),
        ("newline", "nel"),
        ("exit_alt_charset_mode", "rmacs"),
        ("prtr_off", "mc4"),
        ("enter_leftward_mode", "slm"),
        ("parm_index", "indn"),
        ("lab_f2", "lf2"),
        ("set_top_margin", "smgt"),
        ("display_pc_char", "dispc"),
        ("key_f62", "kf62"),
        ("restore_cursor", "rc"),
        ("remove_clock", "rmclk"),
        ("repeat_char", "rep"),
        ("key_restart", "krst"),
        ("key_shelp", "kHLP"),
        ("key_sprevious", "kPRV"),
        ("enter_insert_mode", "smir"),
        ("flash_hook", "hook"),
        ("dot_horz_spacing", "spinh"),
        ("define_char", "defc"),
        ("exit_attribute_mode", "sgr0"),
        ("exit_scancode_mode", "rmsc"),
        ("micro_left", "mcub1"),
        ("order_of_pins", "porder"),
        ("max_colors", "colors"),
        ("cpi_changes_res", "cpix"),
        ("key_ll", "kll"),
        ("label_off", "rmln"),
        ("key_mark", "kmrk"),
        ("auto_left_margin", "bw"),
        ("max_pairs", "pairs"),
        ("parm_delete_line", "dl"),
        ("needs_xon_xoff", "nxon"),
        ("key_sleft", "kLFT"),
        ("quick_dial", "qdial"),
        ("dis_status_line", "dsl"),
        ("enter_italics_mode", "sitm"),
        ("dot_vert_spacing", "spinv"),
        ("form_feed", "ff"),
        ("print_rate", "cps"),
        ("get_mouse", "getm"),
        ("key_smessage", "kMSG"),
        ("cursor_to_ll", "ll"),
        ("xon_xoff", "xon"),
        ("exit_pc_charset_mode", "rmpch"),
        ("enter_scancode_mode", "smsc"),
        ("init_1string", "is1"),
        ("row_addr_glitch", "xvpa"),
        ("micro_right", "mcuf1"),
        ("output_res_line", "orl"),
        ("has_print_wheel", "daisy"),
        ("no_pad_char", "npc"),
        ("enter_top_hl_mode", "ethlm"),
        ("clr_bol", "el1"),
        ("key_enter", "kent"),
        ("user2", "u2"),
        ("key_eol", "kel"),
        ("exit_upward_mode", "rum"),
        ("parm_down_cursor", "cud"),
        ("carriage_return", "cr"),
        ("enter_near_letter_quality", "snlq"),
        ("key_scancel", "kCAN"),
        ("set_tab", "hts"),
        ("key_ctab", "kctab"),
        ("subscript_characters", "subcs"),
        ("cursor_left", "cub1"),
        ("pad_char", "pad"),
        ("semi_auto_right_margin", "sam"),
        ("set1_des_seq", "s1ds"),
        ("display_clock", "dclk"),
        ("parm_up_micro", "mcuu"),
        ("dial_phone", "dial"),
        ("key_sundo", "kUND"),
        ("alt_scancode_esc", "scesa"),
        ("enter_left_hl_mode", "elhlm"),
        ("key_sbeg", "kBEG"),
        ("set2_des_seq", "s2ds"),
        ("transparent_underline", "ul"),
        ("stop_bit_image", "rbim"),
        ("key_ic", "kich1"),
        ("lab_f4", "lf4"),
        ("enter_shadow_mode", "sshm"),
        ("lab_f10", "lf10"),
        ("cursor_right", "cuf1"),
        ("bit_image_newline", "binel"),
        ("user6", "u6"),
        ("key_select", "kslt"),
        ("parm_right_cursor", "cuf"),
        ("wait_tone", "wait"),
        ("key_home", "khome"),
        ("set_right_margin", "smgr"),
        ("req_mouse_pos", "reqmp"),
        ("scancode_escape", "scesc"),
        ("set_a_attributes", "sgr1"),
        ("set0_des_seq", "s0ds"),
        ("set_lr_margin", "smglr"),
        ("column_address", "hpa"),
        ("key_open", "kopn"),
        ("erase_overstrike", "eo"),
        ("key_replace", "krpl"),
        ("enter_xon_mode", "smxon"),
        ("enter_doublewide_mode", "swidm"),
        ("prtr_silent", "mc5i"),
        ("enter_vertical_hl_mode", "evhlm"),
        ("user3", "u3"),
        ("key_undo", "kund"),
        ("start_char_set_def", "scsd"),
        ("zero_motion", "zerom"),
        ("key_sdc", "kDC"),
        ("key_end", "kend"),
        ("initialize_color", "initc"),
        ("key_srsume", "kRES"),
        ("key_a1", "ka1"),
        ("has_status_line", "hs"),
        ("buttons", "btns"),
        ("key_c1", "kc1"),
        ("select_char_set", "scs"),
        ("scroll_forward", "ind"),
        ("meta_off", "rmm"),
        ("reset_file", "rf"),
        ("key_catab", "ktbc"),
        ("key_ssuspend", "kSPD"),
        ("key_sprint", "kPRT"),
        ("enter_pc_charset_mode", "smpch"),
        ("set_a_background", "setab"),
        ("insert_character", "ich1"),
        ("init_tabs", "it"),
        ("set_window", "wind"),
        ("key_sfind", "kFND"),
        ("keypad_xmit", "smkx"),
        ("set_color_pair", "scp"),
        ("buffer_capacity", "bufsz"),
        ("insert_padding", "ip"),
        ("set_bottom_margin", "smgb"),
        ("key_sright", "kRIT"),
        ("key_sf", "kind"),
        ("xon_character", "xonc"),
        ("down_half_line", "hd"),
        ("lab_f7", "lf7"),
        ("memory_above", "da"),
        ("exit_ca_mode", "rmcup"),
        ("reset_1string", "rs1"),
        ("memory_below", "db"),
        ("reset_2string", "rs2"),
        ("key_btab", "kcbt"),
        ("key_backspace", "kbs"),
        ("no_color_video", "ncv"),
        ("plab_norm", "pln"),
        ("key_suspend", "kspd"),
        ("parm_right_micro", "mcuf"),
        ("key_right", "kcuf1"),
        ("move_standout_mode", "msgr"),
        ("clr_eos", "ed"),
        ("back_tab", "cbt"),
        ("exit_micro_mode", "rmicm"),
        ("meta_on", "smm"),
        ("req_for_input", "rfi"),
        ("width_status_line", "wsl"),
        ("scroll_reverse", "ri"),
        ("key_snext", "kNXT"),
        ("hard_cursor", "chts"),
        ("cursor_down", "cud1"),
        ("key_save", "ksav"),
        ("hue_lightness_saturation", "hls"),
        ("key_sic", "kIC"),
        ("enter_subscript_mode", "ssubm"),
        ("change_line_pitch", "lpi"),
        ("exit_italics_mode", "ritm"),
        ("erase_chars", "ech"),
        ("orig_colors", "oc"),
        ("key_b2", "kb2"),
        ("ceol_standout_glitch", "xhp"),
        ("non_dest_scroll_region", "ndscr"),
        ("key_soptions", "kOPT"),
        ("auto_right_margin", "am"),
        ("exit_standout_mode", "rmso"),
        ("key_a3", "ka3"),
        ("key_previous", "kprv"),
        ("key_sreplace", "kRPL"),
        ("key_sdl", "kDL"),
        ("cursor_home", "home"),
        ("keypad_local", "rmkx"),
        ("key_find", "kfnd"),
        ("no_esc_ctlc", "xsb"),
        ("virtual_terminal", "vt"),
        ("key_clear", "kclr"),
        ("enter_protected_mode", "prot"),
        ("enter_right_hl_mode", "erhlm"),
        ("enter_low_hl_mode", "elohlm"),
        ("enter_normal_quality", "snrmq"),
        ("key_npage", "knp"),
        ("enter_blink_mode", "blink"),
        ("cursor_invisible", "civis"),
        ("end_bit_image_region", "endbi"),
        ("bit_image_type", "bitype"),
        ("micro_col_size", "mcs"),
        ("set_right_margin_parm", "smgrp"),
        ("key_smove", "kMOV"),
        ("fixed_pause", "pause"),
        ("user7", "u7"),
        ("key_close", "kclo"),
        ("parm_ich", "ich"),
        ("label_height", "lh"),
        ("init_prog", "iprog"),
        ("init_file", "if"),
        ("set_background", "setb"),
        ("orig_pair", "op"),
        ("key_refresh", "krfr"),
        ("change_res_vert", "cvr"),
        ("delete_character", "dch1"),
        ("key_ppage", "kpp"),
        ("device_type", "devt"),
        ("maximum_windows", "wnum"),
        ("these_cause_cr", "docr"),
        ("set_left_margin", "smgl"),
        ("output_res_horz_inch", "orhi"),
        ("key_dc", "kdch1"),
        ("key_c3", "kc3"),
        ("key_print", "kprt"),
        ("cr_cancels_micro_mode", "crxm"),
        ("micro_line_size", "mls"),
        ("key_sexit", "kEXT"),
        ("parm_insert_line", "il"),
        ("reset_3string", "rs3"),
        ("lab_f3", "lf3"),
        ("max_attributes", "ma"),
        ("cursor_up", "cuu1"),
        ("set_foreground", "setf"),
        ("underline_char", "uc"),
        ("start_bit_image", "sbim"),
        ("clear_margins", "mgc"),
        ("bit_image_repeat", "birep"),
        ("change_char_pitch", "cpi"),
        ("delete_line", "dl1"),
        ("row_address", "vpa"),
        ("user0", "u0"),
        ("set_pglen_inch", "slength"),
        ("key_sr", "kri"),
        ("flash_screen", "flash"),
        ("key_stab", "khts"),
        ("key_seol", "kEOL"),
        ("pkey_plab", "pfxl"),
        ("set_left_margin_parm", "smglp"),
        ("columns", "cols"),
        ("pkey_local", "pfloc"),
        ("parm_rindex", "rin"),
        ("user4", "u4"),
        ("user5", "u5"),
        ("max_micro_jump", "mjump"),
        ("wide_char_size", "widcs"),
        ("change_res_horz", "chr"),
        ("tilde_glitch", "hz"),
        ("key_f0", "kf0"),
        ("pkey_key", "pfkey"),
        ("key_move", "kmov"),
        ("cursor_visible", "cvvis"),
        ("lab_f8", "lf8"),
        ("label_on", "smln"),
        ("xoff_character", "xoffc"),
        ("parm_left_cursor", "cub"),
        ("number_of_pins", "npins"),
        ("tone", "tone"),
        ("key_send", "kEND"),
        ("key_up", "kcuu1"),
        ("back_color_erase", "bce"),
        ("key_sredo", "kRDO"),
        ("hard_copy", "hc"),
        ("enter_alt_charset_mode", "smacs"),
        ("parm_down_micro", "mcud"),
        ("key_ssave", "kSAV"),
        ("padding_baud_rate", "pb"),
        ("set_attributes", "sgr"),
        ("key_resume", "kres"),
        ("exit_xon_mode", "rmxon"),
        ("lab_f5", "lf5"),
        ("key_dl", "kdl1"),
        ("lines", "lines"),
        ("command_character", "cmdch"),
        ("key_f63", "kf63"),
        ("lab_f6", "lf6"),
        ("parm_left_micro", "mcub"),
        ("mouse_info", "minfo"),
        ("set_clock", "sclk"),
        ("bit_image_carriage_return", "bicr"),
        ("micro_down", "mcud1"),
        ("has_meta_key", "km"),
        ("prtr_non", "mc5p"),
        ("enter_bold_mode", "bold"),
        ("key_eic", "krmir"),
        ("exit_subscript_mode", "rsubm"),
        ("user1", "u1"),
        ("over_strike", "os"),
        ("set_color_band", "setcolor"),
        ("color_names", "colornm"),
        ("key_copy", "kcpy"),
        ("key_scommand", "kCMD"),
        ("key_cancel", "kcan"),
        ("init_3string", "is3"),
        ("exit_doublewide_mode", "rwidm"),
        ("superscript_characters", "supcs"),
        ("enter_micro_mode", "smicm"),
        ("bell", "bel"),
        ("lab_f0", "lf0"),
        ("dest_tabs_magic_smso", "xt"),
        ("generic_type", "gn"),
        ("pkey_xmit", "pfx"),
        ("clear_screen", "clear"),
        ("set_a_foreground", "setaf"),
        ("set3_des_seq", "s3ds"),
        ("exit_leftward_mode", "rlm"),
        ("code_set_init", "csin"),
        ("enter_reverse_mode", "rev"),
        ("initialize_pair", "initp"),
        ("enter_standout_mode", "smso"),
        ("key_exit", "kext"),
        ("set_top_margin_parm", "smgtp"),
        ("magic_cookie_glitch", "xmc"),
        ("enter_superscript_mode", "ssupm"),
        ("key_il", "kil1"),
        ("set_tb_margin", "smgtb"),
        ("enter_dim_mode", "dim"),
        ("key_beg", "kbeg"),
        ("insert_null_glitch", "in"),
        ("clear_all_tabs", "tbc"),
        ("user8", "u8"),
        ("clr_eol", "el"),
        ("micro_row_address", "mvpa"),
        ("lines_of_memory", "lm"),
        ("exit_superscript_mode", "rsupm"),
        ("lpi_changes_res", "lpix"),
        ("tab", "ht"),
        ("key_help", "khlp"),
        ("enter_horizontal_hl_mode", "ehhlm"),
        ("cursor_normal", "cnorm"),
        ("up_half_line", "hu"),
        ("key_options", "kopt"),
        ("output_res_vert_inch", "orvi"),
        ("enter_ca_mode", "smcup"),
        ("key_left", "kcub1"),
        ("parm_up_cursor", "cuu"),
        ("key_next", "knxt"),
        ("pc_term_options", "pctrm"),
        ("char_set_names", "csnm"),
        ("move_insert_mode", "mir"),
        ("enter_am_mode", "smam"),
        ("to_status_line", "tsl"),
        ("col_addr_glitch", "xhpa"),
        ("lab_f9", "lf9"),
        ("insert_line", "il1"),
        ("exit_shadow_mode", "rshm"),
        ("change_scroll_region", "csr"),
        ("exit_am_mode", "rmam"),
        ("key_eos", "ked"),
        ("key_f1", "kf1"),
        ("enter_underline_mode", "smul"),
        ("lab_f1", "lf1"),
        ("set_page_length", "slines"),
        ("exit_delete_mode", "rmdc"),
        ("char_padding", "rmp"),
        ("exit_insert_mode", "rmir"),
        ("enter_delete_mode", "smdc"),
        ("parm_dch", "dch"),
        ("cursor_mem_address", "mrcup"),
        ("can_change", "ccc"),
        ("acs_chars", "acsc"),
        ("output_res_char", "orc"),
        ("set_bottom_margin_parm", "smgbp"),
        ("key_shome", "kHOM"),
        ("key_mouse", "kmous"),
        ("status_line_esc_ok", "eslok"),
        ("stop_char_set_def", "rcsd"),
        ("label_format", "fln"),
        ("cursor_address", "cup"),
        ("enter_secure_mode", "invis"),
        ("pulse", "pulse"),
        ("label_width", "lw"),
        ("exit_underline_mode", "rmul"),
        ("hangup", "hup"),
        ("key_down", "kcud1"),
        ("key_screate", "kCRT"),
        ("goto_window", "wingo"),
        ("user9", "u9"),
        ("micro_up", "mcuu1"),
        ("key_reference", "kref"),
        ("define_bit_image_region", "defbi"),
        ("non_rev_rmcup", "nrrmc"),
        ("eat_newline_glitch", "xenl"),
        ("key_create", "kcrt"),
        ("num_labels", "nlab"),
        ("enter_draft_quality", "sdrfq"),
        ("create_window", "cwin"),
        ("bit_image_entwining", "bitwin"),
        ("key_command", "kcmd"),
        ("enter_upward_mode", "sum"),
        ("from_status_line", "fsl"),
        ("micro_column_address", "mhpa"),
    ],
};
pub static TERMCAP: ::phf::Map<&'static str, &'static str> = ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 2),
        (0, 0),
        (4, 14),
        (0, 5),
        (0, 12),
        (0, 3),
        (0, 0),
        (0, 2),
        (2, 215),
        (0, 361),
        (0, 158),
        (0, 0),
        (0, 6),
        (0, 9),
        (1, 353),
        (0, 56),
        (3, 23),
        (0, 22),
        (0, 367),
        (2, 132),
        (0, 9),
        (0, 186),
        (1, 148),
        (1, 5),
        (0, 0),
        (0, 7),
        (0, 2),
        (0, 24),
        (0, 162),
        (3, 354),
        (0, 0),
        (1, 25),
        (0, 1),
        (0, 225),
        (1, 131),
        (0, 1),
        (5, 1),
        (0, 103),
        (0, 34),
        (3, 369),
        (0, 0),
        (1, 159),
        (0, 59),
        (2, 89),
        (1, 44),
        (0, 251),
        (0, 66),
        (1, 156),
        (1, 0),
        (0, 29),
        (0, 40),
        (9, 278),
        (1, 260),
        (3, 134),
        (0, 104),
        (0, 168),
        (3, 94),
        (0, 17),
        (0, 7),
        (1, 237),
        (0, 4),
        (0, 107),
        (0, 9),
        (78, 124),
        (0, 13),
        (0, 68),
        (1, 132),
        (15, 382),
        (3, 166),
        (2, 92),
        (0, 107),
        (0, 16),
        (0, 1),
        (53, 366),
        (0, 383),
        (19, 232),
        (243, 339),
    ],
    entries: &[
        ("micro_column_address", "ZY"),
        ("key_a1", "K1"),
        ("key_catab", "ka"),
        ("move_insert_mode", "mi"),
        ("up_half_line", "hu"),
        ("bit_image_newline", "Zz"),
        ("command_character", "CC"),
        ("code_set_init", "ci"),
        ("cpi_changes_res", "YF"),
        ("parm_down_micro", "Zf"),
        ("prtr_on", "po"),
        ("key_ssuspend", "!2"),
        ("key_message", "%3"),
        ("exit_standout_mode", "se"),
        ("cursor_up", "up"),
        ("mouse_info", "Mi"),
        ("col_addr_glitch", "YA"),
        ("key_soptions", "%d"),
        ("these_cause_cr", "Zw"),
        ("key_sfind", "*0"),
        ("key_seol", "*8"),
        ("bit_image_entwining", "Yo"),
        ("insert_character", "ic"),
        ("set2_des_seq", "s2"),
        ("lab_f10", "la"),
        ("max_micro_address", "Yd"),
        ("key_eol", "kE"),
        ("key_b2", "K2"),
        ("newline", "nw"),
        ("label_height", "lh"),
        ("wait_tone", "WA"),
        ("parm_up_micro", "Zi"),
        ("cursor_down", "do"),
        ("set_left_margin", "ML"),
        ("max_pairs", "pa"),
        ("orig_colors", "oc"),
        ("enter_near_letter_quality", "ZK"),
        ("bit_image_type", "Yp"),
        ("key_snext", "%c"),
        ("clr_bol", "cb"),
        ("key_eos", "kS"),
        ("carriage_return", "cr"),
        ("lines", "li"),
        ("enter_insert_mode", "im"),
        ("width_status_line", "ws"),
        ("key_scommand", "*1"),
        ("plab_norm", "pn"),
        ("exit_shadow_mode", "ZU"),
        ("key_home", "kh"),
        ("label_on", "LO"),
        ("key_scancel", "&0"),
        ("enter_standout_mode", "so"),
        ("erase_overstrike", "eo"),
        ("reset_3string", "r3"),
        ("key_srsume", "%j"),
        ("init_1string", "i1"),
        ("xoff_character", "XF"),
        ("exit_alt_charset_mode", "ae"),
        ("transparent_underline", "ul"),
        ("enter_normal_quality", "ZL"),
        ("lab_f6", "l6"),
        ("key_ic", "kI"),
        ("set_window", "wi"),
        ("set_tab", "st"),
        ("key_ctab", "kt"),
        ("key_sdc", "*4"),
        ("user9", "u9"),
        ("cursor_to_ll", "ll"),
        ("key_mouse", "Km"),
        ("prtr_off", "pf"),
        ("get_mouse", "Gm"),
        ("key_sic", "#3"),
        ("row_address", "cv"),
        ("key_send", "*7"),
        ("key_backspace", "kb"),
        ("reset_1string", "r1"),
        ("max_colors", "Co"),
        ("bit_image_repeat", "Xy"),
        ("set_bottom_margin_parm", "Zl"),
        ("magic_cookie_glitch", "sg"),
        ("micro_right", "Zb"),
        ("set_left_margin_parm", "Zm"),
        ("flash_hook", "fh"),
        ("exit_doublewide_mode", "ZQ"),
        ("pad_char", "pc"),
        ("key_reference", "&1"),
        ("num_labels", "Nl"),
        ("key_smessage", "%a"),
        ("key_f62", "Fq"),
        ("prtr_silent", "5i"),
        ("enter_am_mode", "SA"),
        ("enter_alt_charset_mode", "as"),
        ("generic_type", "gn"),
        ("back_color_erase", "ut"),
        ("exit_leftward_mode", "ZS"),
        ("exit_upward_mode", "ZX"),
        ("meta_on", "mm"),
        ("micro_row_address", "Zc"),
        ("req_for_input", "RF"),
        ("enter_xon_mode", "SX"),
        ("set_tb_margin", "MT"),
        ("key_npage", "kN"),
        ("lab_f0", "l0"),
        ("hard_copy", "hc"),
        ("set_right_margin", "MR"),
        ("lab_f1", "l1"),
        ("char_padding", "rP"),
        ("exit_attribute_mode", "me"),
        ("alt_scancode_esc", "S8"),
        ("clear_screen", "cl"),
        ("key_sprint", "%f"),
        ("cursor_right", "nd"),
        ("buffer_capacity", "Ya"),
        ("set_color_pair", "sp"),
        ("key_sbeg", "&9"),
        ("set_a_foreground", "AF"),
        ("key_btab", "kB"),
        ("enter_pc_charset_mode", "S2"),
        ("key_c3", "K5"),
        ("wide_char_size", "Yn"),
        ("key_sreplace", "%h"),
        ("micro_line_size", "Yg"),
        ("select_char_set", "Zj"),
        ("print_screen", "ps"),
        ("enter_subscript_mode", "ZN"),
        ("enter_reverse_mode", "mr"),
        ("device_type", "dv"),
        ("cursor_invisible", "vi"),
        ("set_color_band", "Yz"),
        ("set_top_margin", "Zo"),
        ("ceol_standout_glitch", "xs"),
        ("enter_underline_mode", "us"),
        ("label_width", "lw"),
        ("columns", "co"),
        ("has_status_line", "hs"),
        ("key_f1", "k1"),
        ("exit_xon_mode", "RX"),
        ("enter_bold_mode", "md"),
        ("key_scopy", "*2"),
        ("key_dl", "kL"),
        ("init_prog", "iP"),
        ("parm_right_cursor", "RI"),
        ("no_color_video", "NC"),
        ("change_res_vert", "ZD"),
        ("user8", "u8"),
        ("key_down", "kd"),
        ("initialize_pair", "Ip"),
        ("enter_upward_mode", "ZP"),
        ("parm_dch", "DC"),
        ("auto_left_margin", "bw"),
        ("scroll_reverse", "sr"),
        ("key_il", "kA"),
        ("user6", "u6"),
        ("key_f0", "k0"),
        ("key_refresh", "&2"),
        ("key_screate", "*3"),
        ("micro_left", "Za"),
        ("cursor_normal", "ve"),
        ("lab_f9", "l9"),
        ("key_sr", "kR"),
        ("parm_left_micro", "Zg"),
        ("key_shome", "#2"),
        ("save_cursor", "sc"),
        ("orig_pair", "op"),
        ("key_sdl", "*5"),
        ("init_3string", "i3"),
        ("repeat_char", "rp"),
        ("row_addr_glitch", "YD"),
        ("pc_term_options", "S6"),
        ("insert_null_glitch", "in"),
        ("dot_horz_spacing", "Yc"),
        ("set1_des_seq", "s1"),
        ("key_help", "%1"),
        ("exit_scancode_mode", "S5"),
        ("pkey_local", "pl"),
        ("dis_status_line", "ds"),
        ("key_sf", "kF"),
        ("define_bit_image_region", "Yx"),
        ("parm_up_cursor", "UP"),
        ("acs_chars", "ac"),
        ("micro_col_size", "Yf"),
        ("label_format", "Lf"),
        ("exit_superscript_mode", "ZW"),
        ("req_mouse_pos", "RQ"),
        ("key_ssave", "!1"),
        ("key_open", "%6"),
        ("key_undo", "&8"),
        ("enter_dim_mode", "mh"),
        ("clr_eos", "cd"),
        ("user7", "u7"),
        ("down_half_line", "hd"),
        ("enter_shadow_mode", "ZM"),
        ("delete_line", "dl"),
        ("superscript_characters", "Zv"),
        ("print_rate", "Ym"),
        ("key_stab", "kT"),
        ("cursor_mem_address", "CM"),
        ("parm_down_cursor", "DO"),
        ("key_sright", "%i"),
        ("key_sundo", "!3"),
        ("change_char_pitch", "ZA"),
        ("hangup", "HU"),
        ("start_char_set_def", "Zr"),
        ("to_status_line", "ts"),
        ("enter_draft_quality", "ZG"),
        ("exit_underline_mode", "ue"),
        ("no_esc_ctlc", "xb"),
        ("set_a_background", "AB"),
        ("keypad_xmit", "ks"),
        ("parm_right_micro", "Zh"),
        ("set_pglen_inch", "YI"),
        ("user1", "u1"),
        ("cursor_home", "ho"),
        ("parm_insert_line", "AL"),
        ("padding_baud_rate", "pb"),
        ("key_left", "kl"),
        ("ena_acs", "eA"),
        ("tone", "TO"),
        ("parm_rindex", "SR"),
        ("initialize_color", "Ic"),
        ("set_top_margin_parm", "Zp"),
        ("key_right", "kr"),
        ("exit_subscript_mode", "ZV"),
        ("init_file", "if"),
        ("subscript_characters", "Zu"),
        ("meta_off", "mo"),
        ("virtual_terminal", "vt"),
        ("key_f63", "Fr"),
        ("change_line_pitch", "ZB"),
        ("output_res_vert_inch", "Yl"),
        ("non_rev_rmcup", "NR"),
        ("pkey_xmit", "px"),
        ("max_attributes", "ma"),
        ("key_move", "%4"),
        ("scancode_escape", "S7"),
        ("prtr_non", "pO"),
        ("over_strike", "os"),
        ("set_foreground", "Sf"),
        ("display_clock", "DK"),
        ("output_res_char", "Yi"),
        ("lab_f7", "l7"),
        ("key_suspend", "&7"),
        ("set_right_margin_parm", "Zn"),
        ("exit_delete_mode", "ed"),
        ("key_resume", "&5"),
        ("restore_cursor", "rc"),
        ("micro_down", "ZZ"),
        ("user0", "u0"),
        ("lab_f5", "l5"),
        ("lab_f2", "l2"),
        ("buttons", "BT"),
        ("key_up", "ku"),
        ("exit_micro_mode", "ZT"),
        ("column_address", "ch"),
        ("key_options", "%7"),
        ("fixed_pause", "PA"),
        ("enter_secure_mode", "mk"),
        ("underline_char", "uc"),
        ("dot_vert_spacing", "Yb"),
        ("goto_window", "WG"),
        ("stop_char_set_def", "Zt"),
        ("key_sprevious", "%e"),
        ("key_next", "%5"),
        ("key_sredo", "%g"),
        ("output_res_horz_inch", "Yk"),
        ("dial_phone", "DI"),
        ("maximum_windows", "MW"),
        ("reset_file", "rf"),
        ("parm_delete_line", "DL"),
        ("enter_protected_mode", "mp"),
        ("display_pc_char", "S1"),
        ("key_sleft", "#4"),
        ("non_dest_scroll_region", "ND"),
        ("clr_eol", "ce"),
        ("semi_auto_right_margin", "YE"),
        ("lab_f4", "l4"),
        ("enter_leftward_mode", "ZI"),
        ("zero_motion", "Zx"),
        ("exit_pc_charset_mode", "S3"),
        ("order_of_pins", "Ze"),
        ("enter_italics_mode", "ZH"),
        ("quick_dial", "QD"),
        ("enter_scancode_mode", "S4"),
        ("cursor_address", "cm"),
        ("enter_micro_mode", "ZJ"),
        ("insert_line", "al"),
        ("scroll_forward", "sf"),
        ("key_smove", "%b"),
        ("output_res_line", "Yj"),
        ("set_bottom_margin", "Zk"),
        ("lab_f8", "l8"),
        ("clear_all_tabs", "ct"),
        ("exit_insert_mode", "ei"),
        ("tab", "ta"),
        ("key_restart", "&4"),
        ("memory_above", "da"),
        ("delete_character", "dc"),
        ("lab_f3", "l3"),
        ("number_of_pins", "Yh"),
        ("key_replace", "&3"),
        ("can_change", "cc"),
        ("erase_chars", "ec"),
        ("pulse", "PU"),
        ("insert_padding", "ip"),
        ("bit_image_carriage_return", "Yv"),
        ("end_bit_image_region", "Yy"),
        ("hard_cursor", "HC"),
        ("tilde_glitch", "hz"),
        ("eat_newline_glitch", "xn"),
        ("back_tab", "bt"),
        ("exit_ca_mode", "te"),
        ("key_c1", "K4"),
        ("set_background", "Sb"),
        ("remove_clock", "RC"),
        ("memory_below", "db"),
        ("key_eic", "kM"),
        ("set_clock", "SC"),
        ("has_print_wheel", "YC"),
        ("form_feed", "ff"),
        ("key_sexit", "*9"),
        ("user3", "u3"),
        ("micro_up", "Zd"),
        ("enter_delete_mode", "dm"),
        ("key_ppage", "kP"),
        ("lpi_changes_res", "YG"),
        ("user2", "u2"),
        ("cursor_visible", "vs"),
        ("keypad_local", "ke"),
        ("user5", "u5"),
        ("key_save", "&6"),
        ("color_names", "Yw"),
        ("init_tabs", "it"),
        ("lines_of_memory", "lm"),
        ("pkey_plab", "xl"),
        ("max_micro_jump", "Ye"),
        ("set_page_length", "YZ"),
        ("start_bit_image", "Zq"),
        ("parm_index", "SF"),
        ("bell", "bl"),
        ("auto_right_margin", "am"),
        ("cr_cancels_micro_mode", "YB"),
        ("stop_bit_image", "Zs"),
        ("key_ll", "kH"),
        ("char_set_names", "Zy"),
        ("user4", "u4"),
        ("key_redo", "%0"),
        ("no_pad_char", "NP"),
        ("change_res_horz", "ZC"),
        ("parm_ich", "IC"),
        ("enter_blink_mode", "mb"),
        ("reset_2string", "r2"),
        ("set_attributes", "sa"),
        ("enter_ca_mode", "ti"),
        ("status_line_esc_ok", "es"),
        ("has_meta_key", "km"),
        ("set3_des_seq", "s3"),
        ("change_scroll_region", "cs"),
        ("key_dc", "kD"),
        ("exit_am_mode", "RA"),
        ("key_clear", "kC"),
        ("hue_lightness_saturation", "hl"),
        ("pkey_key", "pk"),
        ("define_char", "ZE"),
        ("enter_doublewide_mode", "ZF"),
        ("clear_margins", "MC"),
        ("key_previous", "%8"),
        ("key_print", "%9"),
        ("exit_italics_mode", "ZR"),
        ("parm_left_cursor", "LE"),
        ("key_shelp", "#1"),
        ("set0_des_seq", "s0"),
        ("key_mark", "%2"),
        ("flash_screen", "vb"),
        ("dest_tabs_magic_smso", "xt"),
        ("key_a3", "K3"),
        ("init_2string", "is"),
        ("xon_character", "XN"),
        ("move_standout_mode", "ms"),
        ("label_off", "LF"),
        ("from_status_line", "fs"),
        ("cursor_left", "le"),
        ("xon_xoff", "xo"),
        ("key_select", "*6"),
        ("enter_superscript_mode", "ZO"),
        ("needs_xon_xoff", "nx"),
    ],
};
pub static ALIASES: ::phf::Map<&'static str, &'static str> = ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
        (0, 171),
        (0, 12),
        (0, 80),
        (1, 130),
        (0, 38),
        (0, 279),
        (0, 10),
        (0, 70),
        (0, 739),
        (0, 46),
        (0, 126),
        (0, 15),
        (1, 654),
        (0, 11),
        (0, 1),
        (0, 0),
        (0, 151),
        (0, 150),
        (0, 267),
        (0, 114),
        (0, 0),
        (0, 13),
        (0, 27),
        (0, 0),
        (0, 129),
        (0, 9),
        (0, 52),
        (2, 381),
        (0, 58),
        (0, 1),
        (0, 0),
        (0, 55),
        (0, 12),
        (0, 733),
        (0, 37),
        (0, 6),
        (0, 0),
        (0, 59),
        (0, 90),
        (0, 216),
        (2, 579),
        (1, 303),
        (0, 59),
        (0, 44),
        (4, 507),
        (0, 433),
        (0, 34),
        (0, 620),
        (6, 655),
        (0, 624),
        (0, 31),
        (2, 213),
        (0, 0),
        (0, 335),
        (0, 17),
        (0, 4),
        (0, 35),
        (0, 274),
        (0, 11),
        (1, 619),
        (0, 265),
        (0, 1),
        (0, 0),
        (0, 6),
        (0, 335),
        (0, 7),
        (0, 0),
        (0, 515),
        (0, 68),
        (1, 137),
        (0, 4),
        (1, 634),
        (0, 15),
        (0, 84),
        (0, 90),
        (0, 259),
        (0, 17),
        (0, 1),
        (0, 1),
        (0, 28),
        (0, 23),
        (0, 511),
        (0, 8),
        (0, 164),
        (0, 559),
        (0, 390),
        (0, 41),
        (0, 265),
        (0, 625),
        (3, 432),
        (0, 1),
        (0, 33),
        (0, 20),
        (0, 158),
        (0, 2),
        (0, 91),
        (0, 12),
        (0, 475),
        (2, 360),
        (2, 427),
        (0, 51),
        (2, 140),
        (0, 158),
        (0, 98),
        (1, 370),
        (3, 424),
        (0, 85),
        (0, 87),
        (0, 5),
        (0, 25),
        (0, 53),
        (0, 312),
        (0, 480),
        (0, 6),
        (4, 316),
        (0, 145),
        (0, 54),
        (3, 303),
        (0, 288),
        (0, 5),
        (3, 514),
        (5, 442),
        (0, 55),
        (2, 60),
        (0, 583),
        (1, 653),
        (2, 315),
        (23, 592),
        (0, 186),
        (0, 92),
        (3, 512),
        (0, 24),
        (11, 420),
        (1, 224),
        (0, 0),
        (0, 84),
        (7, 174),
        (0, 0),
        (0, 660),
        (0, 14),
        (9, 66),
        (0, 6),
        (25, 463),
        (3, 224),
        (28, 59),
        (0, 407),
        (0, 0),
        (0, 0),
    ],
    entries: &[
        ("kd", "key_down"),
        ("rP", "char_padding"),
        ("*3", "key_screate"),
        ("msgr", "move_standout_mode"),
        ("kbs", "key_backspace"),
        ("LE", "parm_left_cursor"),
        ("Zq", "start_bit_image"),
        ("erhlm", "enter_right_hl_mode"),
        ("op", "orig_pair"),
        ("kEOL", "key_seol"),
        ("&7", "key_suspend"),
        ("im", "enter_insert_mode"),
        ("orc", "output_res_char"),
        ("%d", "key_soptions"),
        ("s0", "set0_des_seq"),
        ("pfx", "pkey_xmit"),
        ("rmxon", "exit_xon_mode"),
        ("hz", "tilde_glitch"),
        ("cpi", "change_char_pitch"),
        ("nxon", "needs_xon_xoff"),
        ("smglr", "set_lr_margin"),
        ("!3", "key_sundo"),
        ("ZG", "enter_draft_quality"),
        ("px", "pkey_xmit"),
        ("mi", "move_insert_mode"),
        ("rsupm", "exit_superscript_mode"),
        ("kcpy", "key_copy"),
        ("hup", "hangup"),
        ("&2", "key_refresh"),
        ("ZB", "change_line_pitch"),
        ("l7", "lab_f7"),
        ("porder", "order_of_pins"),
        ("kD", "key_dc"),
        ("MW", "maximum_windows"),
        ("ti", "enter_ca_mode"),
        ("mhpa", "micro_column_address"),
        ("K5", "key_c3"),
        ("hl", "hue_lightness_saturation"),
        ("sgr0", "exit_attribute_mode"),
        ("nw", "newline"),
        ("mh", "enter_dim_mode"),
        ("xo", "xon_xoff"),
        ("kEXT", "key_sexit"),
        ("il1", "insert_line"),
        ("%6", "key_open"),
        ("kPRV", "key_sprevious"),
        ("cd", "clr_eos"),
        ("rp", "repeat_char"),
        ("K4", "key_c1"),
        ("lm", "lines_of_memory"),
        ("ut", "back_color_erase"),
        ("rum", "exit_upward_mode"),
        ("tsl", "to_status_line"),
        ("5i", "prtr_silent"),
        ("al", "insert_line"),
        ("sshm", "enter_shadow_mode"),
        ("ethlm", "enter_top_hl_mode"),
        ("K3", "key_a3"),
        ("smln", "label_on"),
        ("kdch1", "key_dc"),
        ("initc", "initialize_color"),
        ("bl", "bell"),
        ("ka", "key_catab"),
        ("kmrk", "key_mark"),
        ("l3", "lab_f3"),
        ("is3", "init_3string"),
        ("Za", "micro_left"),
        ("Yj", "output_res_line"),
        ("RQ", "req_mouse_pos"),
        ("kopn", "key_open"),
        ("iP", "init_prog"),
        ("%b", "key_smove"),
        ("Yy", "end_bit_image_region"),
        ("ktbc", "key_catab"),
        ("csin", "code_set_init"),
        ("spinv", "dot_vert_spacing"),
        ("l2", "lab_f2"),
        ("l1", "lab_f1"),
        ("Zw", "these_cause_cr"),
        ("ka1", "key_a1"),
        ("is2", "init_2string"),
        ("Yh", "number_of_pins"),
        ("ZI", "enter_leftward_mode"),
        ("XF", "xoff_character"),
        ("cup", "cursor_address"),
        ("ZS", "exit_leftward_mode"),
        ("kcmd", "key_command"),
        ("DC", "parm_dch"),
        ("dc", "delete_character"),
        ("Ye", "max_micro_jump"),
        ("Yw", "color_names"),
        ("sgr", "set_attributes"),
        ("rmam", "exit_am_mode"),
        ("ZH", "enter_italics_mode"),
        ("%8", "key_previous"),
        ("%i", "key_sright"),
        ("lf8", "lab_f8"),
        ("ma", "max_attributes"),
        ("ts", "to_status_line"),
        ("kclr", "key_clear"),
        ("docr", "these_cause_cr"),
        ("AL", "parm_insert_line"),
        ("kpp", "key_ppage"),
        ("bold", "enter_bold_mode"),
        ("lines", "lines"),
        ("kbeg", "key_beg"),
        ("ND", "non_dest_scroll_region"),
        ("xmc", "magic_cookie_glitch"),
        ("xt", "dest_tabs_magic_smso"),
        ("bce", "back_color_erase"),
        ("%2", "key_mark"),
        ("km", "has_meta_key"),
        ("mcs", "micro_col_size"),
        ("orhi", "output_res_horz_inch"),
        ("mc5", "prtr_on"),
        ("*4", "key_sdc"),
        ("rmacs", "exit_alt_charset_mode"),
        ("kcud1", "key_down"),
        ("ehhlm", "enter_horizontal_hl_mode"),
        ("so", "enter_standout_mode"),
        ("bt", "back_tab"),
        ("el1", "clr_bol"),
        ("rs2", "reset_2string"),
        ("smgbp", "set_bottom_margin_parm"),
        ("SA", "enter_am_mode"),
        ("Sf", "set_foreground"),
        ("nrrmc", "non_rev_rmcup"),
        ("ue", "exit_underline_mode"),
        ("ZO", "enter_superscript_mode"),
        ("hts", "set_tab"),
        ("is", "init_2string"),
        ("xvpa", "row_addr_glitch"),
        ("lf5", "lab_f5"),
        ("lpi", "change_line_pitch"),
        ("s3ds", "set3_des_seq"),
        ("ndscr", "non_dest_scroll_region"),
        ("%0", "key_redo"),
        ("Yv", "bit_image_carriage_return"),
        ("rmln", "label_off"),
        ("kSAV", "key_ssave"),
        ("kN", "key_npage"),
        ("kA", "key_il"),
        ("ae", "exit_alt_charset_mode"),
        ("hpa", "column_address"),
        ("ul", "transparent_underline"),
        ("smgr", "set_right_margin"),
        ("cuu1", "cursor_up"),
        ("pf", "prtr_off"),
        ("setcolor", "set_color_band"),
        ("md", "enter_bold_mode"),
        ("kcuu1", "key_up"),
        ("kmov", "key_move"),
        ("prot", "enter_protected_mode"),
        ("vs", "cursor_visible"),
        ("&5", "key_resume"),
        ("cm", "cursor_address"),
        ("kf63", "key_f63"),
        ("nd", "cursor_right"),
        ("kSPD", "key_ssuspend"),
        ("knp", "key_npage"),
        ("mcud1", "micro_down"),
        ("Km", "key_mouse"),
        ("ll", "cursor_to_ll"),
        ("Ym", "print_rate"),
        ("krst", "key_restart"),
        ("kDL", "key_sdl"),
        ("DL", "parm_delete_line"),
        ("rmicm", "exit_micro_mode"),
        ("pl", "pkey_local"),
        ("Yc", "dot_horz_spacing"),
        ("mk", "enter_secure_mode"),
        ("pb", "padding_baud_rate"),
        ("*0", "key_sfind"),
        ("Zz", "bit_image_newline"),
        ("smpch", "enter_pc_charset_mode"),
        ("k1", "key_f1"),
        ("gn", "generic_type"),
        ("YF", "cpi_changes_res"),
        ("sitm", "enter_italics_mode"),
        ("i3", "init_3string"),
        ("dv", "device_type"),
        ("dim", "enter_dim_mode"),
        ("me", "exit_attribute_mode"),
        ("ZR", "exit_italics_mode"),
        ("kMSG", "key_smessage"),
        ("#4", "key_sleft"),
        ("xsb", "no_esc_ctlc"),
        ("sum", "enter_upward_mode"),
        ("Yx", "define_bit_image_region"),
        ("defc", "define_char"),
        ("wi", "set_window"),
        ("#3", "key_sic"),
        ("kFND", "key_sfind"),
        ("S5", "exit_scancode_mode"),
        ("rs1", "reset_1string"),
        ("lf10", "lab_f10"),
        ("xoffc", "xoff_character"),
        ("l0", "lab_f0"),
        ("mgc", "clear_margins"),
        ("kRES", "key_srsume"),
        ("*2", "key_scopy"),
        ("kslt", "key_select"),
        ("!1", "key_ssave"),
        ("kres", "key_resume"),
        ("K1", "key_a1"),
        ("setb", "set_background"),
        ("le", "cursor_left"),
        ("slength", "set_pglen_inch"),
        ("Yl", "output_res_vert_inch"),
        ("ke", "keypad_local"),
        ("setaf", "set_a_foreground"),
        ("lf4", "lab_f4"),
        ("tbc", "clear_all_tabs"),
        ("kCAN", "key_scancel"),
        ("ZZ", "micro_down"),
        ("l4", "lab_f4"),
        ("ML", "set_left_margin"),
        ("khlp", "key_help"),
        ("Zh", "parm_right_micro"),
        ("mls", "micro_line_size"),
        ("hd", "down_half_line"),
        ("kLFT", "key_sleft"),
        ("cmdch", "command_character"),
        ("Zy", "char_set_names"),
        ("mc5i", "prtr_silent"),
        ("indn", "parm_index"),
        ("*6", "key_select"),
        ("cwin", "create_window"),
        ("kNXT", "key_snext"),
        ("smacs", "enter_alt_charset_mode"),
        ("scs", "select_char_set"),
        ("PU", "pulse"),
        ("kf1", "key_f1"),
        ("kHOM", "key_shome"),
        ("S2", "enter_pc_charset_mode"),
        ("mcud", "parm_down_micro"),
        ("smgt", "set_top_margin"),
        ("nlab", "num_labels"),
        ("rshm", "exit_shadow_mode"),
        ("krmir", "key_eic"),
        ("rwidm", "exit_doublewide_mode"),
        ("*9", "key_sexit"),
        ("smgrp", "set_right_margin_parm"),
        ("hook", "flash_hook"),
        ("ZQ", "exit_doublewide_mode"),
        ("QD", "quick_dial"),
        ("xn", "eat_newline_glitch"),
        ("ZJ", "enter_micro_mode"),
        ("minfo", "mouse_info"),
        ("l5", "lab_f5"),
        ("sbim", "start_bit_image"),
        ("enacs", "ena_acs"),
        ("YC", "has_print_wheel"),
        ("Yb", "dot_vert_spacing"),
        ("subcs", "subscript_characters"),
        ("cuu", "parm_up_cursor"),
        ("chr", "change_res_horz"),
        ("kprv", "key_previous"),
        ("it", "init_tabs"),
        ("nel", "newline"),
        ("S1", "display_pc_char"),
        ("binel", "bit_image_newline"),
        ("kh", "key_home"),
        ("i1", "init_1string"),
        ("colornm", "color_names"),
        ("ZU", "exit_shadow_mode"),
        ("xhpa", "col_addr_glitch"),
        ("rmdc", "exit_delete_mode"),
        ("mb", "enter_blink_mode"),
        ("dl1", "delete_line"),
        ("if", "init_file"),
        ("cr", "carriage_return"),
        ("Zi", "parm_up_micro"),
        ("Zu", "subscript_characters"),
        ("kctab", "key_ctab"),
        ("getm", "get_mouse"),
        ("hs", "has_status_line"),
        ("uc", "underline_char"),
        ("Yn", "wide_char_size"),
        ("#2", "key_shome"),
        ("mp", "enter_protected_mode"),
        ("s1ds", "set1_des_seq"),
        ("xenl", "eat_newline_glitch"),
        ("smm", "meta_on"),
        ("defbi", "define_bit_image_region"),
        ("as", "enter_alt_charset_mode"),
        ("Zr", "start_char_set_def"),
        ("kS", "key_eos"),
        ("ZM", "enter_shadow_mode"),
        ("eslok", "status_line_esc_ok"),
        ("ku", "key_up"),
        ("el", "clr_eol"),
        ("u3", "user3"),
        ("LO", "label_on"),
        ("Ic", "initialize_color"),
        ("kcub1", "key_left"),
        ("us", "enter_underline_mode"),
        ("rmcup", "exit_ca_mode"),
        ("pfloc", "pkey_local"),
        ("l6", "lab_f6"),
        ("YG", "lpi_changes_res"),
        ("cs", "change_scroll_region"),
        ("rmso", "exit_standout_mode"),
        ("kf0", "key_f0"),
        ("%c", "key_snext"),
        ("vt", "virtual_terminal"),
        ("SF", "parm_index"),
        ("ci", "code_set_init"),
        ("db", "memory_below"),
        ("xonc", "xon_character"),
        ("r3", "reset_3string"),
        ("ssupm", "enter_superscript_mode"),
        ("sr", "scroll_reverse"),
        ("qdial", "quick_dial"),
        ("am", "auto_right_margin"),
        ("slm", "enter_leftward_mode"),
        ("xhp", "ceol_standout_glitch"),
        ("hls", "hue_lightness_saturation"),
        ("kc1", "key_c1"),
        ("kEND", "key_send"),
        ("RX", "exit_xon_mode"),
        ("wait", "wait_tone"),
        ("ZY", "micro_column_address"),
        ("chts", "hard_cursor"),
        ("colors", "max_colors"),
        ("rsubm", "exit_subscript_mode"),
        ("rlm", "exit_leftward_mode"),
        ("u6", "user6"),
        ("mcuu1", "micro_up"),
        ("kspd", "key_suspend"),
        ("ic", "insert_character"),
        ("kopt", "key_options"),
        ("orl", "output_res_line"),
        ("pO", "prtr_non"),
        ("up", "cursor_up"),
        ("S6", "pc_term_options"),
        ("u0", "user0"),
        ("npins", "number_of_pins"),
        ("smgtp", "set_top_margin_parm"),
        ("lf9", "lab_f9"),
        ("acsc", "acs_chars"),
        ("da", "memory_above"),
        ("ZP", "enter_upward_mode"),
        ("bitype", "bit_image_type"),
        ("&1", "key_reference"),
        ("cub1", "cursor_left"),
        ("SX", "enter_xon_mode"),
        ("ht", "tab"),
        ("CC", "command_character"),
        ("%3", "key_message"),
        ("Ip", "initialize_pair"),
        ("ws", "width_status_line"),
        ("*1", "key_scommand"),
        ("rmul", "exit_underline_mode"),
        ("kE", "key_eol"),
        ("elhlm", "enter_left_hl_mode"),
        ("ce", "clr_eol"),
        ("rmclk", "remove_clock"),
        ("%g", "key_sredo"),
        ("cb", "clr_bol"),
        ("kUND", "key_sundo"),
        ("kil1", "key_il"),
        ("ct", "clear_all_tabs"),
        ("wingo", "goto_window"),
        ("u5", "user5"),
        ("crxm", "cr_cancels_micro_mode"),
        ("kdl1", "key_dl"),
        ("!2", "key_ssuspend"),
        ("pln", "plab_norm"),
        ("kext", "key_exit"),
        ("kf62", "key_f62"),
        ("cnorm", "cursor_normal"),
        ("evhlm", "enter_vertical_hl_mode"),
        ("kF", "key_sf"),
        ("reqmp", "req_mouse_pos"),
        ("Gm", "get_mouse"),
        ("IC", "parm_ich"),
        ("kb2", "key_b2"),
        ("PA", "fixed_pause"),
        ("cub", "parm_left_cursor"),
        ("kCMD", "key_scommand"),
        ("RC", "remove_clock"),
        ("kPRT", "key_sprint"),
        ("rmpch", "exit_pc_charset_mode"),
        ("RA", "exit_am_mode"),
        ("sf", "scroll_forward"),
        ("MR", "set_right_margin"),
        ("Yf", "micro_col_size"),
        ("smso", "enter_standout_mode"),
        ("%j", "key_srsume"),
        ("ei", "exit_insert_mode"),
        ("kBEG", "key_sbeg"),
        ("birep", "bit_image_repeat"),
        ("kH", "key_ll"),
        ("pn", "plab_norm"),
        ("bicr", "bit_image_carriage_return"),
        ("kref", "key_reference"),
        ("ind", "scroll_forward"),
        ("khome", "key_home"),
        ("kL", "key_dl"),
        ("eA", "ena_acs"),
        ("xl", "pkey_plab"),
        ("smul", "enter_underline_mode"),
        ("lf0", "lab_f0"),
        ("SR", "parm_rindex"),
        ("pa", "max_pairs"),
        ("ZE", "define_char"),
        ("dsl", "dis_status_line"),
        ("ms", "move_standout_mode"),
        ("bitwin", "bit_image_entwining"),
        ("mjump", "max_micro_jump"),
        ("vi", "cursor_invisible"),
        ("rmsc", "exit_scancode_mode"),
        ("khts", "key_stab"),
        ("Ze", "order_of_pins"),
        ("ec", "erase_chars"),
        ("civis", "cursor_invisible"),
        ("rmp", "char_padding"),
        ("sdrfq", "enter_draft_quality"),
        ("daisy", "has_print_wheel"),
        ("ve", "cursor_normal"),
        ("WG", "goto_window"),
        ("kr", "key_right"),
        ("bel", "bell"),
        ("ed", "clr_eos"),
        ("%h", "key_sreplace"),
        ("smcup", "enter_ca_mode"),
        ("Zj", "select_char_set"),
        ("kcrt", "key_create"),
        ("BT", "buttons"),
        ("slines", "set_page_length"),
        ("MT", "set_tb_margin"),
        ("fln", "label_format"),
        ("ssubm", "enter_subscript_mode"),
        ("CM", "cursor_mem_address"),
        ("te", "exit_ca_mode"),
        ("mcub1", "micro_left"),
        ("ho", "cursor_home"),
        ("smxon", "enter_xon_mode"),
        ("kind", "key_sf"),
        ("YE", "semi_auto_right_margin"),
        ("mcuf", "parm_right_micro"),
        ("kcuf1", "key_right"),
        ("smgl", "set_left_margin"),
        ("se", "exit_standout_mode"),
        ("pad", "pad_char"),
        ("YB", "cr_cancels_micro_mode"),
        ("mvpa", "micro_row_address"),
        ("rmm", "meta_off"),
        ("kmous", "key_mouse"),
        ("pctrm", "pc_term_options"),
        ("sg", "magic_cookie_glitch"),
        ("do", "cursor_down"),
        ("lpix", "lpi_changes_res"),
        ("k0", "key_f0"),
        ("Lf", "label_format"),
        ("dial", "dial_phone"),
        ("*8", "key_seol"),
        ("ri", "scroll_reverse"),
        ("pc", "pad_char"),
        ("invis", "enter_secure_mode"),
        ("rep", "repeat_char"),
        ("kcbt", "key_btab"),
        ("orvi", "output_res_vert_inch"),
        ("&4", "key_restart"),
        ("%9", "key_print"),
        ("kund", "key_undo"),
        ("kMOV", "key_smove"),
        ("cl", "clear_screen"),
        ("btns", "buttons"),
        ("cud", "parm_down_cursor"),
        ("cols", "columns"),
        ("in", "insert_null_glitch"),
        ("Yd", "max_micro_address"),
        ("kM", "key_eic"),
        ("rev", "enter_reverse_mode"),
        ("kc3", "key_c3"),
        ("Ya", "buffer_capacity"),
        ("kri", "key_sr"),
        ("kB", "key_btab"),
        ("ZN", "enter_subscript_mode"),
        ("Zs", "stop_bit_image"),
        ("spinh", "dot_horz_spacing"),
        ("LF", "label_off"),
        ("pfkey", "pkey_key"),
        ("TO", "tone"),
        ("mcuu", "parm_up_micro"),
        ("elohlm", "enter_low_hl_mode"),
        ("ked", "key_eos"),
        ("&3", "key_replace"),
        ("es", "status_line_esc_ok"),
        ("&8", "key_undo"),
        ("mo", "meta_off"),
        ("HU", "hangup"),
        ("DO", "parm_down_cursor"),
        ("kOPT", "key_soptions"),
        ("krfr", "key_refresh"),
        ("kel", "key_eol"),
        ("vpa", "row_address"),
        ("sam", "semi_auto_right_margin"),
        ("devt", "device_type"),
        ("ka3", "key_a3"),
        ("fsl", "from_status_line"),
        ("NR", "non_rev_rmcup"),
        ("u1", "user1"),
        ("rin", "parm_rindex"),
        ("&6", "key_save"),
        ("l8", "lab_f8"),
        ("Yg", "micro_line_size"),
        ("xb", "no_esc_ctlc"),
        ("rmkx", "keypad_local"),
        ("sclk", "set_clock"),
        ("mcub", "parm_left_micro"),
        ("lf7", "lab_f7"),
        ("fs", "from_status_line"),
        ("xon", "xon_xoff"),
        ("YD", "row_addr_glitch"),
        ("rmir", "exit_insert_mode"),
        ("Fq", "key_f62"),
        ("cuf", "parm_right_cursor"),
        ("YZ", "set_page_length"),
        ("mc4", "prtr_off"),
        ("pk", "pkey_key"),
        ("kent", "key_enter"),
        ("ps", "print_screen"),
        ("S8", "alt_scancode_esc"),
        ("is1", "init_1string"),
        ("Nl", "num_labels"),
        ("ccc", "can_change"),
        ("clear", "clear_screen"),
        ("MC", "clear_margins"),
        ("kI", "key_ic"),
        ("RI", "parm_right_cursor"),
        ("*5", "key_sdl"),
        ("dclk", "display_clock"),
        ("mm", "meta_on"),
        ("ks", "keypad_xmit"),
        ("kl", "key_left"),
        ("kHLP", "key_shelp"),
        ("Fr", "key_f63"),
        ("rs3", "reset_3string"),
        ("Yz", "set_color_band"),
        ("smgtb", "set_tb_margin"),
        ("s3", "set3_des_seq"),
        ("ich", "parm_ich"),
        ("kRDO", "key_sredo"),
        ("%4", "key_move"),
        ("ZK", "enter_near_letter_quality"),
        ("smir", "enter_insert_mode"),
        ("Zg", "parm_left_micro"),
        ("ZD", "change_res_vert"),
        ("Zt", "stop_char_set_def"),
        ("smgb", "set_bottom_margin"),
        ("rfi", "req_for_input"),
        ("pfxl", "pkey_plab"),
        ("Yo", "bit_image_entwining"),
        ("sgr1", "set_a_attributes"),
        ("XN", "xon_character"),
        ("kend", "key_end"),
        ("Mi", "mouse_info"),
        ("HC", "hard_cursor"),
        ("%f", "key_sprint"),
        ("s2ds", "set2_des_seq"),
        ("Co", "max_colors"),
        ("Yk", "output_res_horz_inch"),
        ("dm", "enter_delete_mode"),
        ("sp", "set_color_pair"),
        ("u2", "user2"),
        ("S3", "exit_pc_charset_mode"),
        ("snrmq", "enter_normal_quality"),
        ("csnm", "char_set_names"),
        ("ZV", "exit_subscript_mode"),
        ("Zo", "set_top_margin"),
        ("S7", "scancode_escape"),
        ("kRIT", "key_sright"),
        ("%5", "key_next"),
        ("smkx", "keypad_xmit"),
        ("st", "set_tab"),
        ("kR", "key_sr"),
        ("Zv", "superscript_characters"),
        ("smsc", "enter_scancode_mode"),
        ("os", "over_strike"),
        ("flash", "flash_screen"),
        ("scesa", "alt_scancode_esc"),
        ("nx", "needs_xon_xoff"),
        ("kprt", "key_print"),
        ("Zn", "set_right_margin_parm"),
        ("u9", "user9"),
        ("setab", "set_a_background"),
        ("RF", "req_for_input"),
        ("%7", "key_options"),
        ("cbt", "back_tab"),
        ("YA", "col_addr_glitch"),
        ("Xy", "bit_image_repeat"),
        ("kb", "key_backspace"),
        ("scp", "set_color_pair"),
        ("u8", "user8"),
        ("sc", "save_cursor"),
        ("setf", "set_foreground"),
        ("smglp", "set_left_margin_parm"),
        ("kcan", "key_cancel"),
        ("ncv", "no_color_video"),
        ("li", "lines"),
        ("u7", "user7"),
        ("fh", "flash_hook"),
        ("ds", "dis_status_line"),
        ("AB", "set_a_background"),
        ("cps", "print_rate"),
        ("eo", "erase_overstrike"),
        ("AF", "set_a_foreground"),
        ("Zx", "zero_motion"),
        ("u4", "user4"),
        ("bw", "auto_left_margin"),
        ("mir", "move_insert_mode"),
        ("ch", "column_address"),
        ("scsd", "start_char_set_def"),
        ("initp", "initialize_pair"),
        ("ip", "insert_padding"),
        ("s1", "set1_des_seq"),
        ("NC", "no_color_video"),
        ("csr", "change_scroll_region"),
        ("mc5p", "prtr_non"),
        ("ritm", "exit_italics_mode"),
        ("Zl", "set_bottom_margin_parm"),
        ("kfnd", "key_find"),
        ("blink", "enter_blink_mode"),
        ("tone", "tone"),
        ("wsl", "width_status_line"),
        ("kclo", "key_close"),
        ("%1", "key_help"),
        ("vb", "flash_screen"),
        ("ich1", "insert_character"),
        ("kCPY", "key_scopy"),
        ("dl", "parm_delete_line"),
        ("cc", "can_change"),
        ("DK", "display_clock"),
        ("lw", "label_width"),
        ("ZC", "change_res_horz"),
        ("mc0", "print_screen"),
        ("hu", "up_half_line"),
        ("r2", "reset_2string"),
        ("ech", "erase_chars"),
        ("kC", "key_clear"),
        ("&9", "key_sbeg"),
        ("s0ds", "set0_des_seq"),
        ("Zc", "micro_row_address"),
        ("kmsg", "key_message"),
        ("ZA", "change_char_pitch"),
        ("dch", "parm_dch"),
        ("widcs", "wide_char_size"),
        ("ZF", "enter_doublewide_mode"),
        ("zerom", "zero_motion"),
        ("co", "columns"),
        ("dch1", "delete_character"),
        ("ff", "form_feed"),
        ("supcs", "superscript_characters"),
        ("cv", "row_address"),
        ("dispc", "display_pc_char"),
        ("rf", "reset_file"),
        ("knxt", "key_next"),
        ("krpl", "key_replace"),
        ("kt", "key_ctab"),
        ("mcuf1", "micro_right"),
        ("smdc", "enter_delete_mode"),
        ("r1", "reset_1string"),
        ("lf1", "lab_f1"),
        ("kll", "key_ll"),
        ("smicm", "enter_micro_mode"),
        ("cuf1", "cursor_right"),
        ("%a", "key_smessage"),
        ("sa", "set_attributes"),
        ("il", "parm_insert_line"),
        ("kRPL", "key_sreplace"),
        ("cud1", "cursor_down"),
        ("UP", "parm_up_cursor"),
        ("Zb", "micro_right"),
        ("Zk", "set_bottom_margin"),
        ("wnum", "maximum_windows"),
        ("oc", "orig_colors"),
        ("rc", "restore_cursor"),
        ("ta", "tab"),
        ("lf3", "lab_f3"),
        ("mr", "enter_reverse_mode"),
        ("DI", "dial_phone"),
        ("la", "lab_f10"),
        ("ZT", "exit_micro_mode"),
        ("pulse", "pulse"),
        ("krdo", "key_redo"),
        ("bufsz", "buffer_capacity"),
        ("po", "prtr_on"),
        ("endbi", "end_bit_image_region"),
        ("pairs", "max_pairs"),
        ("ac", "acs_chars"),
        ("lf2", "lab_f2"),
        ("lh", "label_height"),
        ("%e", "key_sprevious"),
        ("smam", "enter_am_mode"),
        ("xs", "ceol_standout_glitch"),
        ("Yp", "bit_image_type"),
        ("*7", "key_send"),
        ("s2", "set2_des_seq"),
        ("ZW", "exit_superscript_mode"),
        ("kP", "key_ppage"),
        ("ZL", "enter_normal_quality"),
        ("kCRT", "key_screate"),
        ("Zf", "parm_down_micro"),
        ("iprog", "init_prog"),
        ("kich1", "key_ic"),
        ("rbim", "stop_bit_image"),
        ("Sb", "set_background"),
        ("Zd", "micro_up"),
        ("NP", "no_pad_char"),
        ("S4", "enter_scancode_mode"),
        ("&0", "key_scancel"),
        ("swidm", "enter_doublewide_mode"),
        ("kT", "key_stab"),
        ("kIC", "key_sic"),
        ("home", "cursor_home"),
        ("npc", "no_pad_char"),
        ("maddr", "max_micro_address"),
        ("hc", "hard_copy"),
        ("cvr", "change_res_vert"),
        ("pause", "fixed_pause"),
        ("ZX", "exit_upward_mode"),
        ("Yi", "output_res_char"),
        ("lf6", "lab_f6"),
        ("rcsd", "stop_char_set_def"),
        ("#1", "key_shelp"),
        ("SC", "set_clock"),
        ("wind", "set_window"),
        ("YI", "set_pglen_inch"),
        ("K2", "key_b2"),
        ("l9", "lab_f9"),
        ("WA", "wait_tone"),
        ("ksav", "key_save"),
        ("cpix", "cpi_changes_res"),
        ("Zm", "set_left_margin_parm"),
        ("snlq", "enter_near_letter_quality"),
        ("scesc", "scancode_escape"),
        ("cvvis", "cursor_visible"),
        ("mrcup", "cursor_mem_address"),
        ("Zp", "set_top_margin_parm"),
        ("kDC", "key_sdc"),
    ],
};
